@@ -3,12 +3,13 @@
 Covertype dataset with dense SGD
 ================================
 
-Benchmark stochastic gradient descent (SGD), Liblinear, and Naive Bayes on
-the forest covertype dataset of Blackard, Jock, and Dean [1]. The dataset
-comprises 581,012 samples. It is low-dimensional with 54 features and a
-sparsity of approx. 23%. Here, we consider the task of predicting class 1
-(spruce/fir). The classification performance of SGD is competitive with
-Liblinear while being two orders of magnitude faster to train::
+Benchmark stochastic gradient descent (SGD), Liblinear, and Naive Bayes, CART
+(decision tree), RandomForest and Extra-Trees on the forest covertype dataset
+of Blackard, Jock, and Dean [1]. The dataset comprises 581,012 samples. It is
+low dimensional with 54 features and a sparsity of approx. 23%. Here, we
+consider the task of predicting class 1 (spruce/fir). The classification
+performance of SGD is competitive with Liblinear while being two orders of
+magnitude faster to train::
 
     [..]
     Classification performance:
@@ -16,10 +17,12 @@ Liblinear while being two orders of magnitude faster to train::
 
     Classifier   train-time test-time error-rate
     --------------------------------------------
-    Liblinear     9.4471s    0.0184s     0.2305
-    GaussianNB           2.5426s    0.1725s     0.3633
-    SGD           0.2137s    0.0047s     0.2300
-
+    Liblinear     11.8977s   0.0285s     0.2305
+    GaussianNB    3.5931s    0.6645s     0.3633
+    SGD           0.2924s    0.0114s     0.2300
+    CART          39.9829s   0.0345s     0.0476
+    RandomForest  794.6232s  1.0526s     0.0249
+    Extra-Trees   1401.7051s 1.1181s     0.0230
 
 The same task has been used in a number of papers including:
 
@@ -39,7 +42,7 @@ The same task has been used in a number of papers including:
 
 To run this example use your favorite python shell::
 
-  % ipython examples/sgd/covertype_dense_sgd.py
+  % ipython benchmark/bench_sgd_covertype.py
 
 """
 from __future__ import division
@@ -58,6 +61,8 @@ import numpy as np
 from sklearn.svm import LinearSVC
 from sklearn.linear_model import SGDClassifier
 from sklearn.naive_bayes import GaussianNB
+from sklearn.tree import DecisionTreeClassifier
+from sklearn.ensemble import RandomForestClassifier, ExtraTreesClassifier
 from sklearn import metrics
 
 ######################################################################
@@ -110,8 +115,8 @@ mean = X_train.mean(axis=0)
 std = X_train.std(axis=0)
 mean[10:] = 0.0
 std[10:] = 1.0
-X_train = (X_train-mean) / std
-X_test = (X_test-mean) / std
+X_train = (X_train - mean) / std
+X_test = (X_test - mean) / std
 
 ######################################################################
 ## Print dataset statistics
@@ -123,11 +128,11 @@ print("%s %d" % ("number of features:".ljust(25),
 print("%s %d" % ("number of classes:".ljust(25),
                  np.unique(y_train).shape[0]))
 print("%s %d (%d, %d)" % ("number of train samples:".ljust(25),
-                          X_train.shape[0], np.sum(y_train==1),
-                          np.sum(y_train==-1)))
+                          X_train.shape[0], np.sum(y_train == 1),
+                          np.sum(y_train == -1)))
 print("%s %d (%d, %d)" % ("number of test samples:".ljust(25),
-                          X_test.shape[0], np.sum(y_test==1),
-                          np.sum(y_test==-1)))
+                          X_test.shape[0], np.sum(y_test == 1),
+                          np.sum(y_test == -1)))
 print("")
 print("Training classifiers...")
 print("")
@@ -170,6 +175,26 @@ sgd_parameters = {
 sgd_err, sgd_train_time, sgd_test_time = benchmark(SGDClassifier(
     **sgd_parameters))
 
+######################################################################
+## Train CART model
+cart_err, cart_train_time, cart_test_time = benchmark(
+    DecisionTreeClassifier(min_split=5,
+                           max_depth=None))
+
+######################################################################
+## Train RandomForest model
+rf_err, rf_train_time, rf_test_time = benchmark(
+    RandomForestClassifier(n_estimators=20,
+                           min_split=5,
+                           max_depth=None))
+
+######################################################################
+## Train Extra-Trees model
+et_err, et_train_time, et_test_time = benchmark(
+    ExtraTreesClassifier(n_estimators=20,
+                         min_split=5,
+                         max_depth=None))
+
 ######################################################################
 ## Print classification performance
 print("")
@@ -191,5 +216,8 @@ print_row("Liblinear", liblinear_train_time, liblinear_test_time,
           liblinear_err)
 print_row("GaussianNB", gnb_train_time, gnb_test_time, gnb_err)
 print_row("SGD", sgd_train_time, sgd_test_time, sgd_err)
+print_row("CART", cart_train_time, cart_test_time, cart_err)
+print_row("RandomForest", rf_train_time, rf_test_time, rf_err)
+print_row("Extra-Trees", et_train_time, et_test_time, et_err)
 print("")
 print("")
