@@ -7,15 +7,19 @@ example files.
 Files that generate images should start with 'plot'
 
 """
+from time import time
 import os
 import shutil
 import traceback
 import glob
+import sys
+from StringIO import StringIO
 
 import matplotlib
 matplotlib.use('Agg')
 
-import token, tokenize
+import token
+import tokenize
 
 rst_template = """
 
@@ -37,6 +41,8 @@ plot_rst_template = """
 
 %(image_list)s
 
+%(stdout)s
+
 **Python source code:** :download:`%(fname)s <%(fname)s>`
 
 .. literalinclude:: %(fname)s
@@ -63,6 +69,7 @@ SINGLE_IMAGE = """
     :align: center
 """
 
+
 def extract_docstring(filename):
     """ Extract a module-level docstring, if any
     """
@@ -84,11 +91,11 @@ def extract_docstring(filename):
             # If the docstring is formatted with several paragraphs, extract
             # the first one:
             paragraphs = '\n'.join(line.rstrip()
-                                for line in docstring.split('\n')).split('\n\n')
+                              for line in docstring.split('\n')).split('\n\n')
             if len(paragraphs) > 0:
                 first_par = paragraphs[0]
         break
-    return docstring, first_par, erow+1+start_row
+    return docstring, first_par, erow + 1 + start_row
 
 
 def generate_example_rst(app):
@@ -96,7 +103,7 @@ def generate_example_rst(app):
         examples.
     """
     root_dir = os.path.join(app.builder.srcdir, 'auto_examples')
-    example_dir = os.path.abspath(app.builder.srcdir +  '/../' + 'examples')
+    example_dir = os.path.abspath(app.builder.srcdir + '/../' + 'examples')
     try:
         plot_gallery = eval(app.builder.config.plot_gallery)
     except TypeError:
@@ -155,11 +162,11 @@ def generate_dir_rst(dir, fhindex, example_dir, root_dir, plot_gallery):
         target_dir = root_dir
         src_dir = example_dir
     if not os.path.exists(os.path.join(src_dir, 'README.txt')):
-        print 80*'_'
+        print 80 * '_'
         print ('Example directory %s does not have a README.txt file'
                         % src_dir)
         print 'Skipping this directory'
-        print 80*'_'
+        print 80 * '_'
         return
     fhindex.write("""
 
@@ -185,15 +192,23 @@ def generate_dir_rst(dir, fhindex, example_dir, root_dir, plot_gallery):
             if link_name.startswith('._'):
                 link_name = link_name[2:]
             if dir != '.':
-                fhindex.write('   :target: ./%s/%s.html\n\n' % (dir, fname[:-3]))
+                fhindex.write('   :target: ./%s/%s.html\n\n' % (dir,
+                                                               fname[:-3]))
             else:
                 fhindex.write('   :target: ./%s.html\n\n' % link_name[:-3])
-            fhindex.write('   :ref:`example_%s`\n\n' % link_name)
+            fhindex.write("""   :ref:`example_%s`
+
+.. toctree::
+   :hidden:
+
+   %s/%s
+
+""" % (link_name, dir, fname[:-3]))
     fhindex.write("""
 .. raw:: html
 
     <div style="clear: both"></div>
-    """) # clear at the end of the section
+    """)  # clear at the end of the section
 
 
 def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
@@ -224,26 +239,50 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
     if not os.path.exists(thumb_dir):
         os.makedirs(thumb_dir)
     image_path = os.path.join(image_dir, image_fname)
+    stdout_path = os.path.join(image_dir,
+                               'stdout_%s.txt' % base_image_name)
     thumb_file = os.path.join(thumb_dir, fname[:-3] + '.png')
     if plot_gallery and fname.startswith('plot'):
         # generate the plot as png image if file name
         # starts with plot and if it is more recent than an
         # existing image.
         first_image_file = image_path % 1
+        if os.path.exists(stdout_path):
+            stdout = open(stdout_path).read()
+        else:
+            stdout = ''
 
         if (not os.path.exists(first_image_file) or
                 os.stat(first_image_file).st_mtime <=
                                     os.stat(src_file).st_mtime):
             # We need to execute the code
             print 'plotting %s' % fname
+            t0 = time()
             import matplotlib.pyplot as plt
             plt.close('all')
             cwd = os.getcwd()
             try:
-                # First CD in the original example dir, so that any file created
-                # by the example get created in this directory
+                # First CD in the original example dir, so that any file
+                # created by the example get created in this directory
+                orig_stdout = sys.stdout
                 os.chdir(os.path.dirname(src_file))
-                execfile(os.path.basename(src_file), {'pl' : plt})
+                my_stdout = StringIO()
+                sys.stdout = my_stdout
+                my_globals = {'pl': plt}
+                execfile(os.path.basename(src_file), my_globals)
+                sys.stdout = orig_stdout
+                my_stdout = my_stdout.getvalue()
+                if '__doc__' in my_globals:
+                    # The __doc__ is often printed in the example, we
+                    # don't with to echo it
+                    my_stdout = my_stdout.replace(
+                                            my_globals['__doc__'],
+                                            '')
+                my_stdout = my_stdout.strip()
+                if my_stdout:
+                    stdout = '**Script output**::\n\n  %s\n\n' % (
+                        '\n  '.join(my_stdout.split('\n')))
+                open(stdout_path, 'w').write(stdout)
                 os.chdir(cwd)
 
                 # In order to save every figure we have two solutions :
@@ -253,19 +292,22 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
                 # * iterate over [fig_mngr.num for fig_mngr in
                 #   matplotlib._pylab_helpers.Gcf.get_all_fig_managers()]
                 for fig_num in (fig_mngr.num for fig_mngr in
-                          matplotlib._pylab_helpers.Gcf.get_all_fig_managers()):
+                        matplotlib._pylab_helpers.Gcf.get_all_fig_managers()):
                     # Set the fig_num figure as the current figure as we can't
                     # save a figure that's not the current figure.
                     plt.figure(fig_num)
                     plt.savefig(image_path % fig_num)
                     figure_list.append(image_fname % fig_num)
             except:
-                print 80*'_'
+                print 80 * '_'
                 print '%s is not compiling:' % fname
                 traceback.print_exc()
-                print 80*'_'
+                print 80 * '_'
             finally:
                 os.chdir(cwd)
+                sys.stdout = orig_stdout
+
+            print " - time elapsed : %.2g sec" % (time() - t0)
         else:
             figure_list = [f[len(image_dir):]
                             for f in glob.glob(image_path % '[1-9]')]
@@ -293,7 +335,7 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
         for figure_name in figure_list:
             image_list += HLIST_IMAGE_TEMPLATE % figure_name.lstrip('/')
 
-    f = open(os.path.join(target_dir, fname[:-2] + 'rst'),'w')
+    f = open(os.path.join(target_dir, fname[:-2] + 'rst'), 'w')
     f.write(this_template % locals())
     f.flush()
 
@@ -301,3 +343,24 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
 def setup(app):
     app.connect('builder-inited', generate_example_rst)
     app.add_config_value('plot_gallery', True, 'html')
+
+    # Sphinx hack: sphinx copies generated images to the build directory
+    #  each time the docs are made.  If the desired image name already
+    #  exists, it appends a digit to prevent overwrites.  The problem is,
+    #  the directory is never cleared.  This means that each time you build
+    #  the docs, the number of images in the directory grows.
+    #
+    # This question has been asked on the sphinx development list, but there
+    #  was no response: http://osdir.com/ml/sphinx-dev/2011-02/msg00123.html
+    #
+    # The following is a hack that prevents this behavior by clearing the
+    #  image build directory each time the docs are built.  If sphinx
+    #  changes their layout between versions, this will not work (though
+    #  it should probably not cause a crash).  Tested successfully
+    #  on Sphinx 1.0.7
+    build_image_dir = '_build/html/_images'
+    if os.path.exists(build_image_dir):
+        filelist = os.listdir(build_image_dir)
+        for filename in filelist:
+            if filename.endswith('png'):
+                os.remove(os.path.join(build_image_dir, filename))
