@@ -34,7 +34,7 @@ from sklearn.cross_validation import KFold
 ###############################################################################
 # Generate data
 n_samples = 200
-size = 40 # image size
+size = 40  # image size
 roi_size = 15
 snr = 5.
 np.random.seed(0)
@@ -44,8 +44,8 @@ coef = np.zeros((size, size))
 coef[0:roi_size, 0:roi_size] = -1.
 coef[-roi_size:, -roi_size:] = 1.
 
-X = np.random.randn(n_samples, size**2)
-for x in X: # smooth data
+X = np.random.randn(n_samples, size ** 2)
+for x in X:  # smooth data
     x[:] = ndimage.gaussian_filter(x.reshape(size, size), sigma=1.0).ravel()
 X -= X.mean(axis=0)
 X /= X.std(axis=0)
@@ -53,11 +53,11 @@ X /= X.std(axis=0)
 y = np.dot(X, coef.ravel())
 noise = np.random.randn(y.shape[0])
 noise_coef = (linalg.norm(y, 2) / np.exp(snr / 20.)) / linalg.norm(noise, 2)
-y += noise_coef * noise # add noise
+y += noise_coef * noise  # add noise
 
 ###############################################################################
 # Compute the coefs of a Bayesian Ridge with GridSearch
-cv = KFold(len(y), 2) # cross-validation generator for model selection
+cv = KFold(len(y), 2)  # cross-validation generator for model selection
 ridge = BayesianRidge()
 mem = Memory(cachedir='.', verbose=1)
 
@@ -67,21 +67,21 @@ ward = WardAgglomeration(n_clusters=10, connectivity=A, memory=mem,
                          n_components=1)
 clf = Pipeline([('ward', ward), ('ridge', ridge)])
 # Select the optimal number of parcels with grid search
-clf = GridSearchCV(clf, {'ward__n_clusters': [10, 20, 30]}, n_jobs=1)
-clf.fit(X, y, cv=cv) # set the best parameters
-coef_ = clf.best_estimator.steps[-1][1].coef_
-coef_ = clf.best_estimator.steps[0][1].inverse_transform(coef_)
+clf = GridSearchCV(clf, {'ward__n_clusters': [10, 20, 30]}, n_jobs=1, cv=cv)
+clf.fit(X, y)  # set the best parameters
+coef_ = clf.best_estimator_.steps[-1][1].coef_
+coef_ = clf.best_estimator_.steps[0][1].inverse_transform(coef_)
 coef_agglomeration_ = coef_.reshape(size, size)
 
 # Anova univariate feature selection followed by BayesianRidge
-f_regression = mem.cache(feature_selection.f_regression) # caching function
+f_regression = mem.cache(feature_selection.f_regression)  # caching function
 anova = feature_selection.SelectPercentile(f_regression)
 clf = Pipeline([('anova', anova), ('ridge', ridge)])
 # Select the optimal percentage of features with grid search
-clf = GridSearchCV(clf, {'anova__percentile': [5, 10, 20]})
-clf.fit(X, y, cv=cv) # set the best parameters
-coef_ = clf.best_estimator.steps[-1][1].coef_
-coef_ = clf.best_estimator.steps[0][1].inverse_transform(coef_)
+clf = GridSearchCV(clf, {'anova__percentile': [5, 10, 20]}, cv=cv)
+clf.fit(X, y)  # set the best parameters
+coef_ = clf.best_estimator_.steps[-1][1].coef_
+coef_ = clf.best_estimator_.steps[0][1].inverse_transform(coef_)
 coef_selection_ = coef_.reshape(size, size)
 
 ###############################################################################
