@@ -63,7 +63,7 @@ logging.basicConfig(level=logging.INFO,
                     format='%(asctime)s %(levelname)s %(message)s')
 
 
-################################################################################
+###############################################################################
 # Load some categories from the training set
 categories = [
     'alt.atheism',
@@ -80,7 +80,7 @@ print "%d documents" % len(data.filenames)
 print "%d categories" % len(data.target_names)
 print
 
-################################################################################
+###############################################################################
 # define a pipeline combining a text feature extractor with a simple
 # classifier
 pipeline = Pipeline([
@@ -94,7 +94,7 @@ parameters = {
 # increase processing time in a combinatorial way
     'vect__max_df': (0.5, 0.75, 1.0),
 #    'vect__max_features': (None, 5000, 10000, 50000),
-    'vect__analyzer__max_n': (1, 2), # words or bigrams
+    'vect__analyzer__max_n': (1, 2),  # words or bigrams
 #    'tfidf__use_idf': (True, False),
 #    'tfidf__norm': ('l1', 'l2'),
     'clf__alpha': (0.00001, 0.000001),
@@ -106,16 +106,12 @@ parameters = {
 # classifier
 grid_search = GridSearchCV(pipeline, parameters, n_jobs=1)
 
-# cross-validation doesn't work if the length of the data is not known,
-# hence use lists instead of iterators
-text_docs = [file(f).read() for f in data.filenames]
-
 print "Performing grid search..."
 print "pipeline:", [name for name, _ in pipeline.steps]
 print "parameters:"
 pprint(parameters)
 t0 = time()
-grid_search.fit(text_docs, data.target)
+grid_search.fit(data.data, data.target)
 print "done in %0.3fs" % (time() - t0)
 print
 
