@@ -19,10 +19,9 @@ from sklearn.qda import QDA
 ###############################################################################
 # colormap
 cmap = colors.LinearSegmentedColormap('red_blue_classes',
-    {'red' : [(0, 1, 1), (1, 0.7, 0.7)],
-     'green' : [(0, 0.7, 0.7), (1, 0.7, 0.7)],
-     'blue' : [(0, 0.7, 0.7), (1, 1, 1)]
-    })
+    {'red': [(0, 1, 1), (1, 0.7, 0.7)],
+     'green': [(0, 0.7, 0.7), (1, 0.7, 0.7)],
+     'blue': [(0, 0.7, 0.7), (1, 1, 1)]})
 pl.cm.register_cmap(cmap=cmap)
 
 
@@ -38,6 +37,7 @@ def dataset_fixed_cov():
     y = np.hstack((np.zeros(n), np.ones(n)))
     return X, y
 
+
 def dataset_cov():
     '''Generate 2 Gaussians samples with different covariance matrices'''
     n, dim = 300, 2
@@ -48,6 +48,7 @@ def dataset_cov():
     y = np.hstack((np.zeros(n), np.ones(n)))
     return X, y
 
+
 ###############################################################################
 # plot functions
 def plot_data(lda, X, y, y_pred, fig_index):
@@ -60,7 +61,7 @@ def plot_data(lda, X, y, y_pred, fig_index):
     elif fig_index == 3:
         pl.ylabel('Different covariances')
 
-    tp = (y == y_pred) # True Positive
+    tp = (y == y_pred)  # True Positive
     tp0, tp1 = tp[y == 0], tp[y == 1]
     X0, X1 = X[y == 0], X[y == 1]
     X0_tp, X0_fp = X0[tp0], X0[tp0 != True]
@@ -70,11 +71,11 @@ def plot_data(lda, X, y, y_pred, fig_index):
 
     # class 0: dots
     pl.plot(X0_tp[:, 0], X0_tp[:, 1], 'o', color='red')
-    pl.plot(X0_fp[:, 0], X0_fp[:, 1], '.', color='#990000') # dark red
+    pl.plot(X0_fp[:, 0], X0_fp[:, 1], '.', color='#990000')  # dark red
 
     # class 1: dots
     pl.plot(X1_tp[:, 0], X1_tp[:, 1], 'o', color='blue')
-    pl.plot(X1_fp[:, 0], X1_fp[:, 1], '.', color='#000099') # dark blue
+    pl.plot(X1_fp[:, 0], X1_fp[:, 1], '.', color='#000099')  # dark blue
 
     # class 0 and 1 : areas
     nx, ny = 200, 100
@@ -96,11 +97,12 @@ def plot_data(lda, X, y, y_pred, fig_index):
 
     return splot
 
+
 def plot_ellipse(splot, mean, cov, color):
     v, w = linalg.eigh(cov)
     u = w[0] / linalg.norm(w[0])
-    angle = np.arctan(u[1]/u[0])
-    angle = 180 * angle / np.pi # convert to degrees
+    angle = np.arctan(u[1] / u[0])
+    angle = 180 * angle / np.pi  # convert to degrees
     # filled gaussian at 2 standard deviation
     ell = mpl.patches.Ellipse(mean, 2 * v[0] ** 0.5, 2 * v[1] ** 0.5,
                                             180 + angle, color=color)
@@ -108,10 +110,12 @@ def plot_ellipse(splot, mean, cov, color):
     ell.set_alpha(0.5)
     splot.add_artist(ell)
 
+
 def plot_lda_cov(lda, splot):
     plot_ellipse(splot, lda.means_[0], lda.covariance_, 'red')
     plot_ellipse(splot, lda.means_[1], lda.covariance_, 'blue')
 
+
 def plot_qda_cov(qda, splot):
     plot_ellipse(splot, qda.means_[0], qda.covariances_[0], 'red')
     plot_ellipse(splot, qda.means_[1], qda.covariances_[1], 'blue')
