@@ -13,11 +13,7 @@ machine-learning as a versatile tool for science and engineering.
 See http://scikit-learn.sourceforge.net for complete documentation.
 """
 
-try:
-    from . import check_build
-except ImportError:
-    raise ImportError("Please do not forget to run `make` first")
-
+from . import check_build
 from .base import clone
 
 
@@ -30,6 +26,15 @@ try:
 
         def test(self, label='fast', verbose=1, extra_argv=['--exe'],
                         doctests=True, coverage=False):
+            """Run the full test suite
+
+            Examples
+            --------
+            This will run the test suite and stop at the first failing
+            example
+            >>> from sklearn import test
+            >>> test(extra_argv=['--exe', '-sx']) #doctest: +SKIP
+            """
             return super(NoseTester, self).test(label=label, verbose=verbose,
                                     extra_argv=extra_argv,
                                     doctests=doctests, coverage=coverage)
@@ -47,4 +52,4 @@ __all__ = ['check_build', 'cross_validation', 'cluster', 'covariance',
            'metrics', 'mixture', 'naive_bayes', 'neighbors', 'pipeline',
            'preprocessing', 'qda', 'svm', 'test', 'clone', 'pls']
 
-__version__ = '0.9'
+__version__ = '0.10'
