@@ -6,10 +6,11 @@ Generate samples of synthetic data sets.
 #          G. Louppe
 # License: BSD 3 clause
 
+from itertools import product
 import numpy as np
 from scipy import linalg
 
-from ..utils import check_random_state
+from ..utils import array2d, check_random_state
 
 
 def make_classification(n_samples=100, n_features=20, n_informative=2,
@@ -17,8 +18,7 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
                         n_clusters_per_class=2, weights=None, flip_y=0.01,
                         class_sep=1.0, hypercube=True, shift=0.0, scale=1.0,
                         shuffle=True, random_state=None):
-    """
-    Generate a random n-class classification problem.
+    """Generate a random n-class classification problem.
 
     Parameters
     ----------
@@ -86,8 +86,8 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
         If None, the random number generator is the RandomState instance used
         by `np.random`.
 
-    Return
-    ------
+    Returns
+    -------
     X : array of shape [n_samples, n_features]
         The generated samples.
 
@@ -99,8 +99,8 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
     The algorithm is adapted from Guyon [1] and was designed to generate
     the "Madelon" dataset.
 
-    References
-    ----------
+    **References**:
+
     .. [1] I. Guyon, "Design of experiments for the NIPS 2003 variable
            selection benchmark", 2003.
     """
@@ -136,7 +136,6 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
     y = np.zeros(n_samples)
 
     # Build the polytope
-    from ..utils.fixes import product
     C = np.array(list(product([-class_sep, class_sep], repeat=n_informative)))
 
     if not hypercube:
@@ -224,11 +223,106 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
     return X, y
 
 
+def make_multilabel_classification(n_samples=100, n_features=20, n_classes=5,
+                                   n_labels=2, length=50,
+                                   allow_unlabeled=True, random_state=None):
+    """Generate a random multilabel classification problem.
+
+    For each sample, the generative process is:
+        - pick the number of labels: n ~ Poisson(n_labels)
+        - n times, choose a class c: c ~ Multinomial(theta)
+        - pick the document length: k ~ Poisson(length)
+        - k times, choose a word: w ~ Multinomial(theta_c)
+
+    In the above process, rejection sampling is used to make sure that
+    n is never zero or more than `n_classes`, and that the document length
+    is never zero. Likewise, we reject classes which have already been chosen.
+
+    Parameters
+    ----------
+    n_samples : int, optional (default=100)
+        The number of samples.
+
+    n_features : int, optional (default=20)
+        The total number of features.
+
+    n_classes : int, optional (default=5)
+        The number of classes of the classification problem.
+
+    n_labels : int, optional (default=2)
+        The average number of labels per instance. Number of labels follows
+        a Poisson distribution that never takes the value 0.
+
+    length : int, optional (default=50)
+        Sum of the features (number of words if documents).
+
+    allow_unlabeled : bool, optional (default=True)
+        If ``True``, some instances might not belong to any class.
+
+    random_state : int, RandomState instance or None, optional (default=None)
+        If int, random_state is the seed used by the random number generator;
+        If RandomState instance, random_state is the random number generator;
+        If None, the random number generator is the RandomState instance used
+        by `np.random`.
+
+    Returns
+    -------
+    X : array of shape [n_samples, n_features]
+        The generated samples.
+
+    Y : list of tuples
+        The label sets.
+    """
+    generator = check_random_state(random_state)
+    p_c = generator.rand(n_classes)
+    p_c /= p_c.sum()
+    p_w_c = generator.rand(n_features, n_classes)
+    p_w_c /= np.sum(p_w_c, axis=0)
+
+    def sample_example():
+        _, n_classes = p_w_c.shape
+
+        # pick a nonzero number of labels per document by rejection sampling
+        n = n_classes + 1
+        while (not allow_unlabeled and n == 0) or n > n_classes:
+            n = generator.poisson(n_labels)
+
+        # pick n classes
+        y = []
+        while len(y) != n:
+            # pick a class with probability P(c)
+            c = generator.multinomial(1, p_c).argmax()
+
+            if not c in y:
+                y.append(c)
+
+        # pick a non-zero document length by rejection sampling
+        k = 0
+        while k == 0:
+            k = generator.poisson(length)
+
+        # generate a document of length k words
+        x = np.zeros(n_features, dtype=int)
+        for i in range(k):
+            if len(y) == 0:
+                # if sample does not belong to any class, generate noise word
+                w = generator.randint(n_features)
+            else:
+                # pick a class and generate an appropriate word
+                c = y[generator.randint(len(y))]
+                w = generator.multinomial(1, p_w_c[:, c]).argmax()
+            x[w] += 1
+
+        return x, y
+
+    X, Y = zip(*[sample_example() for i in range(n_samples)])
+    return np.array(X, dtype=np.float64), Y
+
+
 def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
                     effective_rank=None, tail_strength=0.5, noise=0.0,
                     shuffle=True, coef=False, random_state=None):
-    """
-    Generate a random regression problem.
+    """Generate a random regression problem.
 
     The input set can either be well conditioned (by default) or have a low
     rank-fat tail singular profile. See the `make_low_rank_matrix` for
@@ -342,8 +436,7 @@ def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
 
 def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
                center_box=(-10.0, 10.0), shuffle=True, random_state=None):
-    """
-    Generate isotropic Gaussian blobs for clustering.
+    """Generate isotropic Gaussian blobs for clustering.
 
     Parameters
     ----------
@@ -353,7 +446,8 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
     n_features : int, optional (default=2)
         The number of features for each sample.
 
-    centers : int or array of shape [n_centers, n_features], optional (default=3)
+    centers : int or array of shape [n_centers, n_features], optional
+        (default=3)
         The number of centers to generate, or the fixed center locations.
 
     cluster_std: float or sequence of floats, optional (default=1.0)
@@ -372,8 +466,8 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
         If None, the random number generator is the RandomState instance used
         by `np.random`.
 
-    Return
-    ------
+    Returns
+    -------
     X : array of shape [n_samples, n_features]
         The generated samples.
 
@@ -383,7 +477,8 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
     Examples
     --------
     >>> from sklearn.datasets.samples_generator import make_blobs
-    >>> X, y = make_blobs(n_samples=10, centers=3, n_features=2, random_state=0)
+    >>> X, y = make_blobs(n_samples=10, centers=3, n_features=2,
+    ...                   random_state=0)
     >>> X.shape
     (10, 2)
     >>> y
@@ -395,14 +490,14 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
         centers = generator.uniform(center_box[0], center_box[1],
                                     size=(centers, n_features))
     else:
-        centers = np.atleast_2d(centers)
+        centers = array2d(centers)
         n_features = centers.shape[1]
 
     X = []
     y = []
 
     n_centers = centers.shape[0]
-    n_samples_per_center = [n_samples / n_centers] * n_centers
+    n_samples_per_center = [int(n_samples // n_centers)] * n_centers
 
     for i in xrange(n_samples % n_centers):
         n_samples_per_center[i] += 1
@@ -425,9 +520,9 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
 
 
 def make_friedman1(n_samples=100, n_features=10, noise=0.0, random_state=None):
-    """
-    Generate the "Friedman #1" regression problem as described in Friedman [1]
-    and Breiman [2].
+    """Generate the "Friedman #1" regression problem
+
+    This dataset is described in Friedman [1] and Breiman [2].
 
     Inputs `X` are independent features uniformly distributed on the interval
     [0, 1]. The output `y` is created according to the formula::
@@ -465,8 +560,10 @@ def make_friedman1(n_samples=100, n_features=10, noise=0.0, random_state=None):
     y : array of shape [n_samples]
         The output values.
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
+
     .. [1] J. Friedman, "Multivariate adaptive regression splines", The Annals
            of Statistics 19 (1), pages 1-67, 1991.
 
@@ -485,9 +582,9 @@ def make_friedman1(n_samples=100, n_features=10, noise=0.0, random_state=None):
 
 
 def make_friedman2(n_samples=100, noise=0.0, random_state=None):
-    """
-    Generate the "Friedman #2" regression problem as described in Friedman [1]
-    and Breiman [2].
+    """Generate the "Friedman #2" regression problem
+
+    This dataset is described in Friedman [1] and Breiman [2].
 
     Inputs `X` are 4 independent features uniformly distributed on the
     intervals::
@@ -526,8 +623,10 @@ def make_friedman2(n_samples=100, noise=0.0, random_state=None):
     y : array of shape [n_samples]
         The output values.
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
+
     .. [1] J. Friedman, "Multivariate adaptive regression splines", The Annals
            of Statistics 19 (1), pages 1-67, 1991.
 
@@ -551,9 +650,9 @@ def make_friedman2(n_samples=100, noise=0.0, random_state=None):
 
 
 def make_friedman3(n_samples=100, noise=0.0, random_state=None):
-    """
-    Generate the "Friedman #3" regression problem as described in Friedman [1]
-    and Breiman [2].
+    """Generate the "Friedman #3" regression problem
+
+    This dataset is described in Friedman [1] and Breiman [2].
 
     Inputs `X` are 4 independent features uniformly distributed on the
     intervals::
@@ -592,8 +691,10 @@ def make_friedman3(n_samples=100, noise=0.0, random_state=None):
     y : array of shape [n_samples]
         The output values.
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
+
     .. [1] J. Friedman, "Multivariate adaptive regression splines", The Annals
            of Statistics 19 (1), pages 1-67, 1991.
 
@@ -617,9 +718,7 @@ def make_friedman3(n_samples=100, noise=0.0, random_state=None):
 
 def make_low_rank_matrix(n_samples=100, n_features=100, effective_rank=10,
                          tail_strength=0.5, random_state=None):
-    """
-    Generate a mostly low rank random matrix with bell-shaped singular
-    values profile.
+    """Generate a mostly low rank matrix with bell-shaped singular values
 
     Most of the variance can be explained by a bell-shaped curve of width
     effective_rank: the low rank part of the singular values profile is::
@@ -636,7 +735,7 @@ def make_low_rank_matrix(n_samples=100, n_features=100, effective_rank=10,
     components (singular vectors).
 
     This kind of singular profiles is often seen in practice, for instance:
-     - graw level pictures of faces
+     - gray level pictures of faces
      - TF-IDF vectors of text documents crawled from the web
 
     Parameters
@@ -745,9 +844,9 @@ def make_sparse_coded_signal(n_samples, n_components, n_features,
 
 
 def make_sparse_uncorrelated(n_samples=100, n_features=10, random_state=None):
-    """
-    Generate a random regression problem with sparse uncorrelated design as
-    described in Celeux et al [1].::
+    """Generate a random regression problem with sparse uncorrelated design
+
+    This dataset is described in Celeux et al [1]. as::
 
         X ~ N(0, 1)
         y(X) = X[:, 0] + 2 * X[:, 1] - 2 * X[:, 2] - 1.5 * X[:, 3]
@@ -777,8 +876,10 @@ def make_sparse_uncorrelated(n_samples=100, n_features=10, random_state=None):
     y : array of shape [n_samples]
         The output values.
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
+
     .. [1] G. Celeux, M. El Anbari, J.-M. Marin, C. P. Robert,
            "Regularization in regression: comparing Bayesian and frequentist
            methods in a poorly informative situation", 2009.
@@ -795,8 +896,7 @@ def make_sparse_uncorrelated(n_samples=100, n_features=10, random_state=None):
 
 
 def make_spd_matrix(n_dim, random_state=None):
-    """
-    Generate a random symmetric, positive-definite matrix.
+    """Generate a random symmetric, positive-definite matrix.
 
     Parameters
     ----------
@@ -823,9 +923,63 @@ def make_spd_matrix(n_dim, random_state=None):
     return X
 
 
-def make_swiss_roll(n_samples=100, noise=0.0, random_state=None):
+def make_sparse_spd_matrix(dim=1, alpha=0.95, norm_diag=False,
+                           smallest_coef=.1, largest_coef=.9,
+                           random_state=None):
+    """Generate a sparse symetric definite positive matrix.
+
+    Parameters
+    ----------
+    dim: integer, optional (default=1)
+        The size of the random  (matrix to generate.
+
+    alpha: float between 0 and 1, optional (default=0.95)
+        The probability that a coefficient is non zero (see notes).
+
+    random_state : int, RandomState instance or None, optional (default=None)
+        If int, random_state is the seed used by the random number generator;
+        If RandomState instance, random_state is the random number generator;
+        If None, the random number generator is the RandomState instance used
+        by `np.random`.
+
+    Returns
+    -------
+    prec: array of shape = [dim, dim]
+
+    Notes
+    -----
+    The sparsity is actually imposed on the cholesky factor of the matrix.
+    Thus alpha does not translate directly into the filling fraction of
+    the matrix itself.
     """
-    Generate a swiss roll dataset.
+    random_state = check_random_state(random_state)
+
+    chol = -np.eye(dim)
+    aux = random_state.rand(dim, dim)
+    aux[aux < alpha] = 0
+    aux[aux > alpha] = (smallest_coef
+                        + (largest_coef - smallest_coef)
+                          * random_state.rand(np.sum(aux > alpha)))
+    aux = np.tril(aux, k=-1)
+
+    # Permute the lines: we don't want to have assymetries in the final
+    # SPD matrix
+    permutation = random_state.permutation(dim)
+    aux = aux[permutation].T[permutation]
+    chol += aux
+    prec = np.dot(chol.T, chol)
+
+    if norm_diag:
+        d = np.diag(prec)
+        d = 1. / np.sqrt(d)
+        prec *= d
+        prec *= d[:, np.newaxis]
+
+    return prec
+
+
+def make_swiss_roll(n_samples=100, noise=0.0, random_state=None):
+    """Generate a swiss roll dataset.
 
     Parameters
     ----------
@@ -854,8 +1008,8 @@ def make_swiss_roll(n_samples=100, noise=0.0, random_state=None):
     -----
     The algorithm is from Marsland [1].
 
-    References
-    ----------
+    **References**:
+
     .. [1] S. Marsland, "Machine Learning: An Algorithmic Perpsective",
            Chapter 10, 2009.
            http://www-ist.massey.ac.nz/smarsland/Code/10/lle.py
@@ -876,8 +1030,7 @@ def make_swiss_roll(n_samples=100, noise=0.0, random_state=None):
 
 
 def make_s_curve(n_samples=100, noise=0.0, random_state=None):
-    """
-    Generate an S curve dataset.
+    """Generate an S curve dataset.
 
     Parameters
     ----------
