@@ -1,8 +1,10 @@
 import numpy as np
 from numpy.testing import assert_equal, assert_approx_equal, \
-                          assert_array_almost_equal, assert_array_less
+                          assert_array_almost_equal
+from nose.tools import assert_true
 
 from .. import make_classification
+from .. import make_multilabel_classification
 from .. import make_regression
 from .. import make_blobs
 from .. import make_friedman1
@@ -31,6 +33,18 @@ def test_make_classification():
     assert_equal(sum(y == 2), 65, "Unexpected number of samples in class #2")
 
 
+def test_make_multilabel_classification():
+    for allow_unlabeled, min_length in zip((True, False), (0, 1)):
+        X, Y = make_multilabel_classification(n_samples=100, n_features=20,
+                                              n_classes=3, random_state=0,
+                                              allow_unlabeled=allow_unlabeled)
+        assert_equal(X.shape, (100, 20), "X shape mismatch")
+        if not allow_unlabeled:
+            assert_equal(max([max(y) for y in Y]), 2)
+        assert_equal(min([len(y) for y in Y]), min_length)
+        assert_true(max([len(y) for y in Y]) <= 3)
+
+
 def test_make_regression():
     X, y, c = make_regression(n_samples=100, n_features=10, n_informative=3,
                               effective_rank=5, coef=True, bias=0.0,
@@ -97,7 +111,7 @@ def test_make_low_rank_matrix():
 
     from numpy.linalg import svd
     u, s, v = svd(X)
-    assert sum(s) - 5 < 0.1, "X rank is not approximately 5"
+    assert_true(sum(s) - 5 < 0.1, "X rank is not approximately 5")
 
 
 def test_make_sparse_coded_signal():
