@@ -4,7 +4,10 @@
 #          Lars Buitinck <L.J.Buitinck@uva.nl>
 #
 # License: BSD Style.
-"""Utilities to build feature vectors from text documents"""
+"""
+The :mod:`sklearn.feature_extraction.text` submodule gathers utilities to
+build feature vectors from text documents.
+"""
 
 import re
 import unicodedata
@@ -14,7 +17,10 @@ from ..base import BaseEstimator, TransformerMixin
 from ..preprocessing import normalize
 from ..utils.fixes import Counter
 
-ENGLISH_STOP_WORDS = set([
+# This list of English stop words is taken from the "Glasgow Information
+# Retrieval Group". The original list can be found at
+# http://ir.dcs.gla.ac.uk/resources/linguistic_utils/stop_words
+ENGLISH_STOP_WORDS = frozenset([
     "a", "about", "above", "across", "after", "afterwards", "again", "against",
     "all", "almost", "alone", "along", "already", "also", "although", "always",
     "am", "among", "amongst", "amoungst", "amount", "an", "and", "another",
@@ -86,7 +92,7 @@ def to_ascii(s):
 
 
 def strip_tags(s):
-    return re.compile(r"<([^>]+)>", flags=re.UNICODE).sub("", s)
+    return re.compile(ur"<([^>]+)>", flags=re.UNICODE).sub(u"", s)
 
 
 class RomanPreprocessor(object):
@@ -102,7 +108,16 @@ class RomanPreprocessor(object):
 
 DEFAULT_PREPROCESSOR = RomanPreprocessor()
 
-DEFAULT_TOKEN_PATTERN = r"\b\w\w+\b"
+DEFAULT_TOKEN_PATTERN = ur"\b\w\w+\b"
+
+
+def _check_stop_list(stop):
+    if stop == "english":
+        return ENGLISH_STOP_WORDS
+    elif isinstance(stop, str) or isinstance(stop, unicode):
+        raise ValueError("not a built-in stop list: %s" % stop)
+    else:               # assume it's a collection
+        return stop
 
 
 class WordNGramAnalyzer(BaseEstimator):
@@ -114,11 +129,15 @@ class WordNGramAnalyzer(BaseEstimator):
       - token extraction using unicode regexp word bounderies for token of
         minimum size of 2 symbols (by default)
       - output token n-grams (unigram only by default)
+
+    The stop words argument may be "english" for a built-in list of English
+    stop words or a collection of strings. Note that stop word filtering is
+    performed after preprocessing, which may include accent stripping.
     """
 
     def __init__(self, charset='utf-8', min_n=1, max_n=1,
                  preprocessor=DEFAULT_PREPROCESSOR,
-                 stop_words=ENGLISH_STOP_WORDS,
+                 stop_words="english",
                  token_pattern=DEFAULT_TOKEN_PATTERN):
         self.charset = charset
         self.stop_words = stop_words
@@ -133,7 +152,7 @@ class WordNGramAnalyzer(BaseEstimator):
             # ducktype for file-like objects
             text_document = text_document.read()
 
-        if isinstance(text_document, str):
+        if isinstance(text_document, bytes):
             text_document = text_document.decode(self.charset, 'ignore')
 
         text_document = self.preprocessor.preprocess(text_document)
@@ -148,7 +167,8 @@ class WordNGramAnalyzer(BaseEstimator):
             original_tokens = tokens
             tokens = []
             n_original_tokens = len(original_tokens)
-            for n in xrange(self.min_n, min(self.max_n + 1, n_original_tokens + 1)):
+            for n in xrange(self.min_n,
+                            min(self.max_n + 1, n_original_tokens + 1)):
                 for i in xrange(n_original_tokens - n + 1):
                     tokens.append(u" ".join(original_tokens[i: i + n]))
 
@@ -169,7 +189,7 @@ class CharNGramAnalyzer(BaseEstimator):
     Because of this, it can be considered a basic morphological analyzer.
     """
 
-    white_spaces = re.compile(r"\s\s+")
+    white_spaces = re.compile(ur"\s\s+")
 
     def __init__(self, charset='utf-8', preprocessor=DEFAULT_PREPROCESSOR,
                  min_n=3, max_n=6):
@@ -184,13 +204,13 @@ class CharNGramAnalyzer(BaseEstimator):
             # ducktype for file-like objects
             text_document = text_document.read()
 
-        if isinstance(text_document, str):
+        if isinstance(text_document, bytes):
             text_document = text_document.decode(self.charset, 'ignore')
 
         text_document = self.preprocessor.preprocess(text_document)
 
         # normalize white spaces
-        text_document = self.white_spaces.sub(" ", text_document)
+        text_document = self.white_spaces.sub(u" ", text_document)
 
         text_len = len(text_document)
         ngrams = []
@@ -318,29 +338,27 @@ class CountVectorizer(BaseEstimator):
         # TODO: parallelize the following loop with joblib?
         # (see XXX up ahead)
         for doc in raw_documents:
-            term_count_current = Counter()
+            term_count_current = Counter(self.analyzer.analyze(doc))
+            term_counts.update(term_count_current)
 
-            for term in self.analyzer.analyze(doc):
-                term_count_current[term] += 1
-                term_counts[term] += 1
-
-            if max_df is not None:
-                for term in term_count_current:
-                    document_counts[term] += 1
+            if max_df < 1.0:
+                document_counts.update(term_count_current.iterkeys())
 
             term_counts_per_doc.append(term_count_current)
 
         n_doc = len(term_counts_per_doc)
 
         # filter out stop words: terms that occur in almost all documents
-        if max_df is not None:
+        if max_df < 1.0:
             max_document_count = max_df * n_doc
             stop_words = set(t for t, dc in document_counts.iteritems()
                                if dc > max_document_count)
+        else:
+            stop_words = set()
 
         # list the terms that should be part of the vocabulary
         if max_features is None:
-            terms = [t for t in term_counts if t not in stop_words]
+            terms = set(term_counts) - stop_words
         else:
             # extract the most frequent terms for the vocabulary
             terms = set()
@@ -377,22 +395,10 @@ class CountVectorizer(BaseEstimator):
 
         # raw_documents is an iterable so we don't know its size in advance
 
-        # result of document conversion to term_count_dict
-        term_counts_per_doc = []
-
         # XXX @larsmans tried to parallelize the following loop with joblib.
         # The result was some 20% slower than the serial version.
-        for doc in raw_documents:
-            term_count_current = Counter()
-
-            for term in self.analyzer.analyze(doc):
-                term_count_current[term] += 1
-
-            term_counts_per_doc.append(term_count_current)
-
-        # now that we know the document we can allocate the vectors matrix at
-        # once and fill it with the term counts collected as a temporary list
-        # of dict
+        term_counts_per_doc = [Counter(self.analyzer.analyze(doc))
+                               for doc in raw_documents]
         return self._term_count_dicts_to_matrix(term_counts_per_doc)
 
     def inverse_transform(self, X):
@@ -407,15 +413,20 @@ class CountVectorizer(BaseEstimator):
         X_inv : list of arrays, len = n_samples
             List of arrays of terms.
         """
-        if type(X) is sp.coo_matrix:    # COO matrix is not indexable
+        if sp.isspmatrix_coo(X):  # COO matrix is not indexable
             X = X.tocsr()
+        elif not sp.issparse(X):
+            # We need to convert X to a matrix, so that the indexing
+            # returns 2D objects
+            X = np.asmatrix(X)
+        n_samples = X.shape[0]
 
         terms = np.array(self.vocabulary.keys())
         indices = np.array(self.vocabulary.values())
         inverse_vocabulary = terms[np.argsort(indices)]
 
-        return [inverse_vocabulary[X[i, :].nonzero()[1]]
-                for i in xrange(X.shape[0])]
+        return [inverse_vocabulary[X[i, :].nonzero()[1]].ravel()
+                for i in xrange(n_samples)]
 
 
 class TfidfTransformer(BaseEstimator, TransformerMixin):
@@ -448,12 +459,16 @@ class TfidfTransformer(BaseEstimator, TransformerMixin):
         extra document was seen containing every term in the collection
         exactly once. Prevents zero divisions.
 
-    References
-    ----------
-    R. Baeza-Yates and B. Ribeiro-Neto (2011). Modern Information Retrieval.
-        Addison Wesley, pp. 68–74.
-    C.D. Manning, H. Schütze and P. Raghavan (2008). Introduction to
-        Information Retrieval. Cambridge University Press, pp. 121–125.
+    Notes
+    -----
+    **References**:
+
+    .. [Yates2011] `R. Baeza-Yates and B. Ribeiro-Neto (2011). Modern
+                   Information Retrieval. Addison Wesley, pp. 68–74.`
+
+    .. [MSR2008] `C.D. Manning, H. Schütze and P. Raghavan (2008). Introduction
+                 to Information Retrieval. Cambridge University Press,
+                 pp. 121–125.`
     """
 
     def __init__(self, norm='l2', use_idf=True, smooth_idf=True):
@@ -497,7 +512,12 @@ class TfidfTransformer(BaseEstimator, TransformerMixin):
         n_samples, n_features = X.shape
 
         if self.use_idf:
-            d = sp.lil_matrix((len(self.idf_), len(self.idf_)))
+            expected_n_features = self.idf_.shape[0]
+            if n_features != expected_n_features:
+                raise ValueError("Input has n_features=%d while the model"
+                                 " has been trained with n_features=%d" % (
+                                     n_features, expected_n_features))
+            d = sp.lil_matrix((n_features, n_features))
             d.setdiag(self.idf_)
             # *= doesn't work
             X = X * d
@@ -528,7 +548,7 @@ class Vectorizer(BaseEstimator):
         self.tfidf.fit(X)
         return self
 
-    def fit_transform(self, raw_documents):
+    def fit_transform(self, raw_documents, y=None):
         """
         Learn the representation and return the vectors.
 
