@@ -15,17 +15,20 @@ from scipy import linalg, interpolate
 from scipy.linalg.lapack import get_lapack_funcs
 
 from .base import LinearModel
-from ..utils import arrayfuncs
-from ..utils import deprecated
+from ..utils import array2d, arrayfuncs, deprecated
 from ..cross_validation import check_cv
 from ..externals.joblib import Parallel, delayed
 
 
 def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
-              alpha_min=0, method='lar', overwrite_X=False,
+              alpha_min=0, method='lar', copy_X=True,
               eps=np.finfo(np.float).eps,
-              overwrite_Gram=False, verbose=False):
-    """Compute Least Angle Regression and LASSO path
+              copy_Gram=True, verbose=False):
+    """Compute Least Angle Regression and Lasso path
+
+    The optimization objective for Lasso is::
+
+    (1 / (2 * n_samples)) * ||y - Xw||^2_2 + alpha * ||w||_1
 
     Parameters
     -----------
@@ -69,10 +72,12 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
     See also
     --------
-    :ref:`LassoLars`
-    :ref:`Lars`
-    decomposition.sparse_encode
-    decomposition.sparse_encode_parallel
+    lasso_path
+    LassoLars
+    Lars
+    LassoLarsCV
+    LarsCV
+    sklearn.decomposition.sparse_encode
 
     Notes
     ------
@@ -97,10 +102,10 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
     # referenced.
     L = np.empty((max_features, max_features), dtype=X.dtype)
     swap, nrm2 = linalg.get_blas_funcs(('swap', 'nrm2'), (X,))
-    potrs, = get_lapack_funcs(('potrs',), (X,))
+    solve_cholesky, = get_lapack_funcs(('potrs',), (X,))
 
     if Gram is None:
-        if not overwrite_X:
+        if copy_X:
             # force copy. setting the array to be fortran-ordered
             # speeds up the calculation of the (partial) Gram matrix
             # and allows to easily swap columns
@@ -109,8 +114,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
         Gram = None
         if X.shape[0] > X.shape[1]:
             Gram = np.dot(X.T, X)
-    else:
-        if not overwrite_Gram:
+    elif copy_Gram:
             Gram = Gram.copy()
 
     if Xy is None:
@@ -121,7 +125,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
     if verbose:
         print "Step\t\tAdded\t\tDropped\t\tActive set size\t\tC"
 
-    while 1:
+    while True:
         if Cov.size:
             C_idx = np.argmax(np.abs(Cov))
             C_ = Cov[C_idx]
@@ -147,13 +151,14 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
         if not drop:
 
-            # Update the Cholesky factorization of (Xa * Xa') #
-            #                                                 #
-            #            ( L   0 )                            #
-            #     L  ->  (       )  , where L * w = b         #
-            #            ( w   z )    z = 1 - ||w||           #
-            #                                                 #
-            #   where u is the last added to the active set   #
+            ##########################################################
+            # Append x_j to the Cholesky factorization of (Xa * Xa') #
+            #                                                        #
+            #            ( L   0 )                                   #
+            #     L  ->  (       )  , where L * w = Xa' x_j          #
+            #            ( w   z )    and z = ||x_j||                #
+            #                                                        #
+            ##########################################################
 
             sign_active[n_active] = np.sign(C_)
             m, n = n_active, C_idx + n_active
@@ -180,7 +185,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                                         L[n_active, :n_active])
             v = np.dot(L[n_active, :n_active], L[n_active, :n_active])
             diag = max(np.sqrt(np.abs(c - v)), eps)
-            L[n_active,  n_active] = diag
+            L[n_active, n_active] = diag
 
             active.append(indices[n_active])
             n_active += 1
@@ -190,7 +195,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                                                             n_active, C)
 
         # least squares solution
-        least_squares, info = potrs(L[:n_active, :n_active],
+        least_squares, info = solve_cholesky(L[:n_active, :n_active],
                                sign_active[:n_active], lower=True)
 
         # is this really needed ?
@@ -216,7 +221,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
 
         # TODO: better names for these variables: z
         drop = False
-        z = - coefs[n_iter, active] / least_squares
+        z = -coefs[n_iter, active] / least_squares
         z_pos = arrayfuncs.min_pos(z)
         if z_pos < gamma_:
             # some coefficients have changed sign
@@ -256,7 +261,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                 # propagate dropped variable
                 for i in range(idx, n_active):
                     X.T[i], X.T[i + 1] = swap(X.T[i], X.T[i + 1])
-                    indices[i], indices[i + 1] =  \
+                    indices[i], indices[i + 1] = \
                             indices[i + 1], indices[i]  # yeah this is stupid
 
                 # TODO: this could be updated
@@ -267,7 +272,7 @@ def lars_path(X, y, Xy=None, Gram=None, max_iter=500,
                 Cov = np.r_[temp, Cov]
             else:
                 for i in range(idx, n_active):
-                    indices[i], indices[i + 1] =  \
+                    indices[i], indices[i + 1] = \
                                 indices[i + 1], indices[i]
                     Gram[i], Gram[i + 1] = swap(Gram[i], Gram[i + 1])
                     Gram[:, i], Gram[:, i + 1] = swap(Gram[:, i],
@@ -324,9 +329,8 @@ class Lars(LinearModel):
         calculations. If set to 'auto' let us decide. The Gram
         matrix can also be passed as argument.
 
-    overwrite_X : boolean, optional
-        If True, X will not be copied
-        Default is False
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     eps: float, optional
         The machine-precision regularization in the computation of the
@@ -348,24 +352,23 @@ class Lars(LinearModel):
     --------
     >>> from sklearn import linear_model
     >>> clf = linear_model.Lars(n_nonzero_coefs=1)
-    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1.1111, 0, -1.1111]) # doctest: +ELLIPSIS
-    Lars(eps=..., fit_intercept=True, n_nonzero_coefs=1,
-       normalize=True, overwrite_X=False, precompute='auto', verbose=False)
+    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1.1111, 0, -1.1111])
+    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    Lars(copy_X=True, eps=..., fit_intercept=True, n_nonzero_coefs=1,
+       normalize=True, precompute='auto', verbose=False)
     >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0. -1.11...]
 
-    References
-    ----------
-    http://en.wikipedia.org/wiki/Least_angle_regression
-
     See also
     --------
-    lars_path, LassoLARS, LarsCV, LassoLarsCV
-    decomposition.sparse_encode, decomposition.sparse_encode_parallel
+    lars_path, LarsCV
+    sklearn.decomposition.sparse_encode
+
+    http://en.wikipedia.org/wiki/Least_angle_regression
     """
     def __init__(self, fit_intercept=True, verbose=False, normalize=True,
                  precompute='auto', n_nonzero_coefs=500,
-                 eps=np.finfo(np.float).eps, overwrite_X=False):
+                 eps=np.finfo(np.float).eps, copy_X=True):
         self.fit_intercept = fit_intercept
         self.verbose = verbose
         self.normalize = normalize
@@ -373,7 +376,7 @@ class Lars(LinearModel):
         self.precompute = precompute
         self.n_nonzero_coefs = n_nonzero_coefs
         self.eps = eps
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
 
     def _get_gram(self):
         # precompute if n_samples > n_features
@@ -387,12 +390,12 @@ class Lars(LinearModel):
             Gram = None
         return Gram
 
-    def fit(self, X, y, overwrite_X=False):
+    def fit(self, X, y):
         """Fit the model using X, y as training data.
 
         parameters
         ----------
-        x : array-like, shape = [n_samples, n_features]
+        X : array-like, shape = [n_samples, n_features]
             training data.
 
         y : array-like, shape = [n_samples]
@@ -403,14 +406,13 @@ class Lars(LinearModel):
         self : object
             returns an instance of self.
         """
-
-        X = np.atleast_2d(X)
-        y = np.atleast_1d(y)
+        X = array2d(X)
+        y = np.asarray(y)
 
         X, y, X_mean, y_mean, X_std = self._center_data(X, y,
                                                         self.fit_intercept,
                                                         self.normalize,
-                                                        self.overwrite_X)
+                                                        self.copy_X)
         alpha = getattr(self, 'alpha', 0.)
         if hasattr(self, 'n_nonzero_coefs'):
             alpha = 0.  # n_nonzero_coefs parametrization takes priority
@@ -421,8 +423,8 @@ class Lars(LinearModel):
         Gram = self._get_gram()
 
         self.alphas_, self.active_, self.coef_path_ = lars_path(X, y,
-                  Gram=Gram, overwrite_X=self.overwrite_X,
-                  overwrite_Gram=True, alpha_min=alpha,
+                  Gram=Gram, copy_X=self.copy_X,
+                  copy_Gram=False, alpha_min=alpha,
                   method=self.method, verbose=self.verbose,
                   max_iter=max_iter, eps=self.eps)
 
@@ -437,7 +439,10 @@ class LassoLars(Lars):
     """Lasso model fit with Least Angle Regression a.k.a. Lars
 
     It is a Linear Model trained with an L1 prior as regularizer.
-    lasso).
+
+    The optimization objective for Lasso is::
+
+    (1 / (2 * n_samples)) * ||y - Xw||^2_2 + alpha * ||w||_1
 
     Parameters
     ----------
@@ -452,9 +457,8 @@ class LassoLars(Lars):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    overwrite_X : boolean, optional
-        If True, X will not be copied
-        Default is False
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     precompute : True | False | 'auto' | array-like
         Whether to use a precomputed Gram matrix to speed up
@@ -484,25 +488,28 @@ class LassoLars(Lars):
     --------
     >>> from sklearn import linear_model
     >>> clf = linear_model.LassoLars(alpha=0.01)
-    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1, 0, -1]) # doctest: +ELLIPSIS
-    LassoLars(alpha=0.01, eps=..., fit_intercept=True,
-         max_iter=500, normalize=True, overwrite_X=False, precompute='auto',
-         verbose=False)
+    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1, 0, -1])
+    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    LassoLars(alpha=0.01, copy_X=True, eps=..., fit_intercept=True,
+         max_iter=500, normalize=True, precompute='auto', verbose=False)
     >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0.         -0.963257...]
 
-    References
-    ----------
-    http://en.wikipedia.org/wiki/Least_angle_regression
-
     See also
     --------
-    lars_path, Lasso
+    lars_path
+    lasso_path
+    Lasso
+    LassoCV
+    LassoLarsCV
+    sklearn.decomposition.sparse_encode
+
+    http://en.wikipedia.org/wiki/Least_angle_regression
     """
 
     def __init__(self, alpha=1.0, fit_intercept=True, verbose=False,
                  normalize=True, precompute='auto', max_iter=500,
-                 eps=np.finfo(np.float).eps, overwrite_X=False):
+                 eps=np.finfo(np.float).eps, copy_X=True):
         self.alpha = alpha
         self.fit_intercept = fit_intercept
         self.max_iter = max_iter
@@ -510,28 +517,28 @@ class LassoLars(Lars):
         self.normalize = normalize
         self.method = 'lasso'
         self.precompute = precompute
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
         self.eps = eps
 
 
 # Deprecated classes
+@deprecated("Use Lars instead")
 class LARS(Lars):
     pass
-LARS = deprecated("Use Lars instead")(LARS)
 
 
+@deprecated("Use LassoLars instead")
 class LassoLARS(LassoLars):
     pass
-LassoLARS = deprecated("Use LassoLars instead")(LassoLARS)
 
 
 ###############################################################################
 # Cross-validated estimator classes
 
 def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
-                     overwrite_data=False, method='lars', verbose=False,
-                     fit_intercept=True, normalize=True, max_iter=500,
-                     eps=np.finfo(np.float).eps):
+                        copy=True, method='lars', verbose=False,
+                        fit_intercept=True, normalize=True, max_iter=500,
+                        eps=np.finfo(np.float).eps):
     """Compute the residues on left-out data for a full LARS path
 
     Parameters
@@ -548,8 +555,9 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
         Precomputed Gram matrix (X' * X), if 'auto', the Gram
         matrix is precomputed from the given X, if there are more samples
         than features
-    overwrite_data: boolean, optional
-        Whether X_train, X_test, y_train and y_test get overriden
+    copy: boolean, optional
+        Whether X_train, X_test, y_train and y_test should be copied;
+        if False, they may be overwritten.
     method: 'lar' | 'lasso'
         Specifies the returned model. Select 'lar' for Least Angle
         Regression, 'lasso' for the Lasso.
@@ -586,7 +594,7 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
     residues: array, shape (n_features, max_features + 1)
         Residues of the prediction on the test data
     """
-    if not overwrite_data:
+    if copy:
         X_train = X_train.copy()
         y_train = y_train.copy()
         X_test = X_test.copy()
@@ -606,7 +614,7 @@ def _lars_path_residues(X_train, y_train, X_test, y_test, Gram=None,
         X_train[:, nonzeros] /= norms[nonzeros]
 
     alphas, active, coefs = lars_path(X_train, y_train, Gram=Gram,
-                            overwrite_X=True, overwrite_Gram=True,
+                            copy_X=False, copy_Gram=False,
                             method=method, verbose=verbose,
                             max_iter=max_iter, eps=eps)
     if normalize:
@@ -632,9 +640,8 @@ class LarsCV(LARS):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    overwrite_X : boolean, optional
-        If True, X will not be copied
-        Default is False
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     precompute : True | False | 'auto' | array-like
         Whether to use a precomputed Gram matrix to speed up
@@ -678,13 +685,13 @@ class LarsCV(LARS):
 
     def __init__(self, fit_intercept=True, verbose=False, max_iter=500,
                  normalize=True, precompute='auto', cv=None, n_jobs=1,
-                 eps=np.finfo(np.float).eps, overwrite_X=False):
+                 eps=np.finfo(np.float).eps, copy_X=True):
         self.fit_intercept = fit_intercept
         self.max_iter = max_iter
         self.verbose = verbose
         self.normalize = normalize
         self.precompute = precompute
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
         self.cv = cv
         self.n_jobs = n_jobs
         self.eps = eps
@@ -705,9 +712,8 @@ class LarsCV(LARS):
         self : object
             returns an instance of self.
         """
-        X = np.asanyarray(X)
+        X = np.asarray(X)
 
-        n_samples, n_features = X.shape
         # init cross-validation generator
         cv = check_cv(self.cv, X, y, classifier=False)
 
@@ -716,14 +722,14 @@ class LarsCV(LARS):
         cv_paths = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
                     delayed(_lars_path_residues)(X[train], y[train],
                             X[test], y[test], Gram=Gram,
-                            overwrite_data=True, method=self.method,
+                            copy=False, method=self.method,
                             verbose=max(0, self.verbose - 1),
                             normalize=self.normalize,
                             fit_intercept=self.fit_intercept,
                             max_iter=self.max_iter,
                             eps=self.eps)
                     for train, test in cv)
-        all_alphas = np.concatenate(zip(*cv_paths)[0])
+        all_alphas = np.concatenate(list(zip(*cv_paths))[0])
         all_alphas.sort()
 
         mse_path = np.empty((len(all_alphas), len(cv_paths)))
@@ -756,6 +762,10 @@ class LarsCV(LARS):
 class LassoLarsCV(LarsCV):
     """Cross-validated Lasso, using the LARS algorithm
 
+    The optimization objective for Lasso is::
+
+    (1 / (2 * n_samples)) * ||y - Xw||^2_2 + alpha * ||w||_1
+
     Parameters
     ----------
     fit_intercept : boolean
@@ -790,10 +800,8 @@ class LassoLarsCV(LarsCV):
         Cholesky diagonal factors. Increase this for very ill-conditioned
         systems.
 
-    overwrite_X : boolean, optional
-        If True, X will not be copied
-        Default is False
-
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     Attributes
     ----------
@@ -831,7 +839,7 @@ class LassoLarsCV(LarsCV):
 
     See also
     --------
-    lars_path, LassoLARS, LarsCV, LassoCV
+    lars_path, LassoLars, LarsCV, LassoCV
     """
 
     method = 'lasso'
@@ -840,6 +848,10 @@ class LassoLarsCV(LarsCV):
 class LassoLarsIC(LassoLars):
     """Lasso model fit with Lars using BIC or AIC for model selection
 
+    The optimization objective for Lasso is::
+
+    (1 / (2 * n_samples)) * ||y - Xw||^2_2 + alpha * ||w||_1
+
     AIC is the Akaike information criterion and BIC is the Bayes
     Information criterion. Such criteria are useful to select the value
     of the regularization parameter by making a trade-off between the
@@ -862,9 +874,8 @@ class LassoLarsIC(LassoLars):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    overwrite_X : boolean, optional
-        Default is False.
-        If True, X will be overwritten
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     precompute : True | False | 'auto' | array-like
         Whether to use a precomputed Gram matrix to speed up
@@ -891,19 +902,23 @@ class LassoLarsIC(LassoLars):
     `intercept_` : float
         independent term in decision function.
 
+    `alpha_` : float
+        the alpha parameter chosen by the information criterion
+
     Examples
     --------
     >>> from sklearn import linear_model
     >>> clf = linear_model.LassoLarsIC(criterion='bic')
-    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1.1111, 0, -1.1111]) # doctest: +ELLIPSIS
-    LassoLarsIC(criterion='bic', eps=..., fit_intercept=True,
-          max_iter=500, normalize=True, overwrite_X=False, precompute='auto',
+    >>> clf.fit([[-1, 1], [0, 0], [1, 1]], [-1.1111, 0, -1.1111])
+    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    LassoLarsIC(copy_X=True, criterion='bic', eps=..., fit_intercept=True,
+          max_iter=500, normalize=True, precompute='auto',
           verbose=False)
     >>> print clf.coef_ # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
     [ 0.  -1.11...]
 
-    References
-    ----------
+    Notes
+    -----
     The estimation of the number of degrees of freedom is given by:
 
     "On the degrees of freedom of the lasso"
@@ -919,7 +934,7 @@ class LassoLarsIC(LassoLars):
     """
     def __init__(self, criterion='aic', fit_intercept=True, verbose=False,
                  normalize=True, precompute='auto', max_iter=500,
-                 eps=np.finfo(np.float).eps, overwrite_X=False):
+                 eps=np.finfo(np.float).eps, copy_X=True):
         if criterion not in ['aic', 'bic']:
             raise ValueError('criterion should be either bic or aic')
         self.criterion = criterion
@@ -927,11 +942,11 @@ class LassoLarsIC(LassoLars):
         self.max_iter = max_iter
         self.verbose = verbose
         self.normalize = normalize
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
         self.precompute = precompute
         self.eps = eps
 
-    def fit(self, X, y, overwrite_X=False):
+    def fit(self, X, y, copy_X=True):
         """Fit the model using X, y as training data.
 
         parameters
@@ -947,20 +962,20 @@ class LassoLarsIC(LassoLars):
         self : object
             returns an instance of self.
         """
-        X = np.atleast_2d(X)
-        y = np.atleast_1d(y)
+        X = array2d(X)
+        y = np.asarray(y)
 
         X, y, Xmean, ymean, Xstd = LinearModel._center_data(X, y,
                                                     self.fit_intercept,
                                                     self.normalize,
-                                                    self.overwrite_X)
+                                                    self.copy_X)
         max_iter = self.max_iter
 
         Gram = self._get_gram()
 
         alphas_, active_, coef_path_ = lars_path(X, y,
-                  Gram=Gram, overwrite_X=overwrite_X,
-                  overwrite_Gram=True, alpha_min=0.0,
+                  Gram=Gram, copy_X=copy_X,
+                  copy_Gram=False, alpha_min=0.0,
                   method='lasso', verbose=self.verbose,
                   max_iter=max_iter, eps=self.eps)
 
