@@ -9,10 +9,11 @@ import numpy as np
 
 from .base import LinearModel
 from ..utils.extmath import safe_sparse_dot
-from ..utils import safe_asanyarray
+from ..utils import safe_asarray
 from ..preprocessing import LabelBinarizer
 from ..grid_search import GridSearchCV
 
+
 def _solve(A, b, solver, tol):
     # helper method for ridge_regression, A is symmetric positive
 
@@ -46,8 +47,7 @@ def _solve(A, b, solver, tol):
 
 
 def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
-    """
-    Solve the ridge equation by the method of normal equations.
+    """Solve the ridge equation by the method of normal equations.
 
     Parameters
     ----------
@@ -83,7 +83,7 @@ def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
     n_samples, n_features = X.shape
     is_sparse = False
 
-    if hasattr(X, 'todense'): # lazy import of scipy.sparse
+    if hasattr(X, 'todense'):  # lazy import of scipy.sparse
         from scipy import sparse
         is_sparse = sparse.issparse(X)
 
@@ -121,8 +121,11 @@ def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
 
 
 class Ridge(LinearModel):
-    """
-    Ridge regression.
+    """Linear least squares with l2 regularization.
+
+    This model solves a regression model where the loss function is
+    the linear least squares function and regularization is given by
+    the l2-norm. Also known as Ridge Regression or Tikhonov regularization.
 
     Parameters
     ----------
@@ -140,9 +143,8 @@ class Ridge(LinearModel):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    overwrite_X : boolean, optional
-        If True, X will not be copied
-        Default is False
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
 
     tol: float
         Precision of the solution.
@@ -150,9 +152,13 @@ class Ridge(LinearModel):
     Attributes
     ----------
 
-    coef_: array, shape = [n_features] or [n_responses, n_features]
+    `coef_` : array, shape = [n_features] or [n_responses, n_features]
         Weight vector(s).
 
+    See also
+    --------
+    RidgeClassifier, RidgeCV
+
     Examples
     --------
     >>> from sklearn.linear_model import Ridge
@@ -162,22 +168,21 @@ class Ridge(LinearModel):
     >>> y = np.random.randn(n_samples)
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = Ridge(alpha=1.0)
-    >>> clf.fit(X, y)
-    Ridge(alpha=1.0, fit_intercept=True, normalize=False, overwrite_X=False,
+    >>> clf.fit(X, y) # doctest: +NORMALIZE_WHITESPACE
+    Ridge(alpha=1.0, copy_X=True, fit_intercept=True, normalize=False,
        tol=0.001)
     """
 
     def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
-            overwrite_X=False, tol=1e-3):
+            copy_X=True, tol=1e-3):
         self.alpha = alpha
         self.fit_intercept = fit_intercept
         self.normalize = normalize
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
         self.tol = tol
 
     def fit(self, X, y, sample_weight=1.0, solver='auto'):
-        """
-        Fit Ridge regression model
+        """Fit Ridge regression model
 
         Parameters
         ----------
@@ -202,12 +207,12 @@ class Ridge(LinearModel):
         -------
         self : returns an instance of self.
         """
-        X = safe_asanyarray(X, dtype=np.float)
-        y = np.asanyarray(y, dtype=np.float)
+        X = safe_asarray(X, dtype=np.float)
+        y = np.asarray(y, dtype=np.float)
 
         X, y, X_mean, y_mean, X_std = \
            self._center_data(X, y, self.fit_intercept,
-                   self.normalize, self.overwrite_X)
+                   self.normalize, self.copy_X)
 
         self.coef_ = ridge_regression(X, y, self.alpha, sample_weight,
                                       solver, self.tol)
@@ -237,18 +242,21 @@ class RidgeClassifier(Ridge):
     Attributes
     ----------
 
-    coef_: array, shape = [n_features] or [n_classes, n_features]
+    `coef_` : array, shape = [n_features] or [n_classes, n_features]
         Weight vector(s).
 
-    Note
-    ----
+    See also
+    --------
+    Ridge, RidgeClassifierCV
+
+    Notes
+    -----
     For multi-class classification, n_class classifiers are trained in
     a one-versus-all approach.
     """
 
     def fit(self, X, y, solver='auto'):
-        """
-        Fit Ridge regression model.
+        """Fit Ridge regression model.
 
         Parameters
         ----------
@@ -276,11 +284,10 @@ class RidgeClassifier(Ridge):
         return self
 
     def decision_function(self, X):
-        return Ridge.predict(self, X)
+        return Ridge.decision_function(self, X)
 
     def predict(self, X):
-        """
-        Predict target values according to the fitted model.
+        """Predict target values according to the fitted model.
 
         Parameters
         ----------
@@ -288,16 +295,16 @@ class RidgeClassifier(Ridge):
 
         Returns
         -------
-        C : array, shape = [n_samples]
+        y : array, shape = [n_samples]
         """
         Y = self.decision_function(X)
         return self.label_binarizer.inverse_transform(Y)
 
 
 class _RidgeGCV(LinearModel):
-    """
-    Ridge regression with built-in Generalized Cross-Validation, i.e.
-    efficient Leave-One-Out cross-validation.
+    """Ridge regression with built-in Generalized Cross-Validation
+
+    It allows efficient Leave-One-Out cross-validation.
 
     This class is not intended to be used directly. Use RidgeCV instead.
 
@@ -327,21 +334,19 @@ class _RidgeGCV(LinearModel):
 
     looe = y - loov = c / diag(G)
 
-    Reference
-    ---------
-
+    **References**:
     http://cbcl.mit.edu/projects/cbcl/publications/ps/MIT-CSAIL-TR-2007-025.pdf
     http://www.mit.edu/~9.520/spring07/Classes/rlsslides.pdf
     """
 
-    def __init__(self, alphas=[0.1, 1.0, 10.0], fit_intercept=True, normalize=False,
-            score_func=None, loss_func=None, overwrite_X=False):
-        self.alphas = np.asanyarray(alphas)
+    def __init__(self, alphas=[0.1, 1.0, 10.0], fit_intercept=True,
+            normalize=False, score_func=None, loss_func=None, copy_X=True):
+        self.alphas = np.asarray(alphas)
         self.fit_intercept = fit_intercept
         self.normalize = normalize
         self.score_func = score_func
         self.loss_func = loss_func
-        self.overwrite_X = overwrite_X
+        self.copy_X = copy_X
 
     def _pre_compute(self, X, y):
         # even if X is very sparse, K is usually very dense
@@ -395,13 +400,13 @@ class _RidgeGCV(LinearModel):
         -------
         self : Returns self.
         """
-        X = safe_asanyarray(X, dtype=np.float)
-        y = np.asanyarray(y, dtype=np.float)
+        X = safe_asarray(X, dtype=np.float)
+        y = np.asarray(y, dtype=np.float)
 
         n_samples = X.shape[0]
 
         X, y, X_mean, y_mean, X_std = LinearModel._center_data(X, y,
-                self.fit_intercept, self.normalize, self.overwrite_X)
+                self.fit_intercept, self.normalize, self.copy_X)
 
         K, v, Q = self._pre_compute(X, y)
         n_y = 1 if len(y.shape) == 1 else y.shape[1]
@@ -435,8 +440,7 @@ class _RidgeGCV(LinearModel):
 
 
 class RidgeCV(LinearModel):
-    """
-    Ridge regression with built-in cross-validation.
+    """Ridge regression with built-in cross-validation.
 
     By default, it performs Generalized Cross-Validation, which is a form of
     efficient Leave-One-Out cross-validation. Currently, only the n_features >
@@ -445,11 +449,11 @@ class RidgeCV(LinearModel):
     Parameters
     ----------
     alphas: numpy array of shape [n_alpha]
-            Array of alpha values to try.
-            Small positive values of alpha improve the conditioning of the
-            problem and reduce the variance of the estimates.
-            Alpha corresponds to (2*C)^-1 in other linear models such as
-            LogisticRegression or LinearSVC.
+        Array of alpha values to try.
+        Small positive values of alpha improve the conditioning of the
+        problem and reduce the variance of the estimates.
+        Alpha corresponds to (2*C)^-1 in other linear models such as
+        LogisticRegression or LinearSVC.
 
     fit_intercept : boolean
         Whether to calculate the intercept for this model. If set
@@ -471,7 +475,7 @@ class RidgeCV(LinearModel):
 
     See also
     --------
-    Ridge
+    Ridge, RidgeClassifierCV
     """
 
     def __init__(self, alphas=np.array([0.1, 1.0, 10.0]), fit_intercept=True,
@@ -519,8 +523,8 @@ class RidgeCV(LinearModel):
             gs = GridSearchCV(Ridge(fit_intercept=self.fit_intercept),
                               parameters, fit_params=fit_params, cv=self.cv)
             gs.fit(X, y)
-            estimator = gs.best_estimator
-            self.best_alpha = gs.best_estimator.alpha
+            estimator = gs.best_estimator_
+            self.best_alpha = gs.best_estimator_.alpha
 
         self.coef_ = estimator.coef_
         self.intercept_ = estimator.intercept_
@@ -531,8 +535,7 @@ class RidgeCV(LinearModel):
 class RidgeClassifierCV(RidgeCV):
 
     def fit(self, X, y, sample_weight=1.0, class_weight=None):
-        """
-        Fit the ridge classifier.
+        """Fit the ridge classifier.
 
         Parameters
         ----------
@@ -566,8 +569,18 @@ class RidgeClassifierCV(RidgeCV):
         return self
 
     def decision_function(self, X):
-        return RidgeCV.predict(self, X)
+        return RidgeCV.decision_function(self, X)
 
     def predict(self, X):
+        """Predict target values according to the fitted model.
+
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
+
+        Returns
+        -------
+        y : array, shape = [n_samples]
+        """
         Y = self.decision_function(X)
         return self.label_binarizer.inverse_transform(Y)
