@@ -17,16 +17,14 @@ import numpy as np
 from ..utils import check_arrays
 
 
-def unique_labels(*list_of_labels):
-    """Extract an ordered integer array of unique labels
-
-    This implementation ignores any occurrence of NaNs.
-
-    """
-    list_of_labels = [np.unique(labels[np.isfinite(labels)].ravel())
-                      for labels in list_of_labels]
-    list_of_labels = np.concatenate(list_of_labels)
-    return np.unique(list_of_labels)
+def unique_labels(*lists_of_labels):
+    """Extract an ordered array of unique labels"""
+    labels = set()
+    for l in lists_of_labels:
+        if hasattr(l, 'ravel'):
+            l = l.ravel()
+        labels |= set(l)
+    return np.unique(sorted(labels))
 
 
 def confusion_matrix(y_true, y_pred, labels=None):
@@ -49,8 +47,9 @@ def confusion_matrix(y_true, y_pred, labels=None):
     CM : array, shape = [n_classes, n_classes]
         confusion matrix
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
     http://en.wikipedia.org/wiki/Confusion_matrix
 
     """
@@ -94,7 +93,7 @@ def roc_curve(y_true, y_score):
         True Positive Rates
 
     thresholds : array, shape = [>2]
-        Thresholds on proba_ used to compute fpr and tpr
+        Thresholds on y_score used to compute fpr and tpr
 
     Examples
     --------
@@ -106,20 +105,20 @@ def roc_curve(y_true, y_score):
     >>> fpr
     array([ 0. ,  0.5,  0.5,  1. ])
 
-
-    References
-    ----------
+    Notes
+    -----
+    **References**:
     http://en.wikipedia.org/wiki/Receiver_operating_characteristic
 
     """
-    y_true = y_true.ravel()
+    y_true = np.ravel(y_true)
     classes = np.unique(y_true)
 
     # ROC only for binary classification
     if classes.shape[0] != 2:
         raise ValueError("ROC is defined for binary classification only")
 
-    y_score = y_score.ravel()
+    y_score = np.ravel(y_score)
 
     n_pos = float(np.sum(y_true == classes[1]))  # nb of true positive
     n_neg = float(np.sum(y_true == classes[0]))  # nb of true negative
@@ -365,8 +364,9 @@ def f1_score(y_true, y_pred, pos_label=1):
         f1_score of the positive class in binary classification or weighted
         avergage of the f1_scores of each class for the multiclass task
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
     http://en.wikipedia.org/wiki/F1_score
 
     """
@@ -412,8 +412,9 @@ def precision_recall_fscore_support(y_true, y_pred, beta=1.0, labels=None):
     f1_score: array, shape = [n_unique_labels], dtype = np.double
     support: array, shape = [n_unique_labels], dtype = np.long
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
     http://en.wikipedia.org/wiki/Precision_and_recall
 
     """
@@ -562,16 +563,16 @@ def precision_recall_curve(y_true, probas_pred):
         Recall values
 
     thresholds : array, shape = [n]
-        Thresholds on proba_ used to compute precision and recall
+        Thresholds on y_score used to compute precision and recall
 
     """
     y_true = y_true.ravel()
     labels = np.unique(y_true)
     if np.all(labels == np.array([-1, 1])):
         # convert {-1, 1} to boolean {0, 1} repr
+        y_true = y_true.copy()
         y_true[y_true == -1] = 0
-        labels = np.array([0, 1])
-    if not np.all(labels == np.array([0, 1])):
+    elif not np.all(labels == np.array([0, 1])):
         raise ValueError("y_true contains non binary labels: %r" % labels)
 
     probas_pred = probas_pred.ravel()
@@ -580,8 +581,7 @@ def precision_recall_curve(y_true, probas_pred):
     precision = np.empty(n_thresholds)
     recall = np.empty(n_thresholds)
     for i, t in enumerate(thresholds):
-        y_pred = np.ones(len(y_true))
-        y_pred[probas_pred < t] = 0
+        y_pred = (probas_pred >= t).astype(np.int)
         p, r, _, _ = precision_recall_fscore_support(y_true, y_pred)
         precision[i] = p[1]
         recall[i] = r[1]
@@ -624,16 +624,20 @@ def r2_score(y_true, y_pred):
 
     Best possible score is 1.0, lower values are worse.
 
-    Note: not a symmetric function.
-
-    return the R^2 score
-
     Parameters
     ----------
     y_true : array-like
 
     y_pred : array-like
 
+    Returns
+    -------
+    z : float
+        The R^2 score
+
+    Notes
+    -----
+    This is not a symmetric function.
     """
     y_true, y_pred = check_arrays(y_true, y_pred)
     numerator = ((y_true - y_pred) ** 2).sum()
