@@ -10,12 +10,13 @@
 import numpy as np
 from scipy.special import digamma as _digamma, gammaln as _gammaln
 from scipy import linalg
+from scipy.spatial.distance import cdist
 
 from ..utils import check_random_state
 from ..utils.extmath import norm
 from .. import cluster
 from ..metrics import euclidean_distances
-from . gmm import GMM
+from .gmm import GMM
 
 
 def sqnorm(v):
@@ -103,20 +104,21 @@ def _bound_state_loglik_tied(X, initial_bound, bound_prec, precs, means):
     return bound
 
 
+def _sym_quad_form(x, mu, A):
+    """helper function to calculate symmetric quadratic form x.T * A * x"""
+    q = (cdist(x, mu[np.newaxis], "mahalanobis", VI=A) ** 2).reshape(-1)
+    return q
+
+
 def _bound_state_loglik_full(X, initial_bound, bound_prec, precs, means):
     n_components, n_features = means.shape
     n_samples = X.shape[0]
     bound = np.empty((n_samples, n_components))
     bound[:] = bound_prec + initial_bound
     for k in xrange(n_components):
-        d = X - means[k]
-        sqrt_cov = linalg.cholesky(precs[k])
-        d = np.dot(d, sqrt_cov.T)
-        d **= 2
-        bound[:, k] -= 0.5 * d.sum(axis=-1)
+        bound[:, k] -= 0.5 * _sym_quad_form(X, means[k], precs[k])
     return bound
 
-
 _BOUND_STATE_LOGLIK_DICT = dict(
     spherical=_bound_state_loglik_spherical,
     diag=_bound_state_loglik_diag,
@@ -137,7 +139,7 @@ class DPGMM(GMM):
     Stick-breaking Representation of a Gaussian mixture model
     probability distribution. This class allows for easy and efficient
     inference of an approximate posterior distribution over the
-    parameters of a gaussian mixture model with a variable number of
+    parameters of a Gaussian mixture model with a variable number of
     components (smaller than the truncation parameter n_components).
 
     Initialization is with normally-distributed means and identity
@@ -159,7 +161,7 @@ class DPGMM(GMM):
         is as likely to start a new cluster for a point as it is
         to add that point to a cluster with alpha elements. A
         higher alpha means more clusters, as the expected number
-        of clusters is alpha*log(N). Defaults to 1.
+        of clusters is ``alpha*log(N)``. Defaults to 1.
 
     thresh : float, optional
         Convergence threshold.
@@ -185,38 +187,20 @@ class DPGMM(GMM):
     precisions : array
         Precision (inverse covariance) parameters for each mixture
         component.  The shape depends on `cvtype`::
+
             (`n_components`,)                             if 'spherical',
-            (`n_features`, `n_features`)              if 'tied',
+            (`n_features`, `n_features`)                  if 'tied',
             (`n_components`, `n_features`)                if 'diag',
             (`n_components`, `n_features`, `n_features`)  if 'full'
 
-    converged_ : bool
-        True when convergence was reached in fit(), False
-        otherwise.
-
-    Methods
-    -------
-    decode(X)
-        Find most likely mixture components for each point in `X`.
-    eval(X)
-        Compute a lower-bound of the log likelihood of `X` under the model
-        and an approximate posterior distribution over mixture components.
-    fit(X)
-        Estimate the posterior of themodel parameters from `X` using the
-        variational mean-field algorithm.
-    predict(X)
-        Like decode, find most likely mixtures components for each
-        observation in `X`.
-    rvs(n=1)
-        Generate `n` samples from the posterior for the model.
-    score(X)
-        Compute the log likelihood of `X` under the model.
+    `converged_` : bool
+        True when convergence was reached in fit(), False otherwise.
 
     See Also
     --------
-    GMM : Finite gaussian mixture model fit with EM
+    GMM : Finite Gaussian mixture model fit with EM
 
-    VBGMM : Finite gaussian mixture model fit with a variational
+    VBGMM : Finite Gaussian mixture model fit with a variational
     algorithm, better for situations where there might be too little
     data to get a good estimate of the covariance matrix.
 
@@ -281,7 +265,7 @@ class DPGMM(GMM):
             obs = self._X
         else:
             z = np.zeros((obs.shape[0], self.n_components))
-        obs = np.asanyarray(obs)
+        obs = np.asarray(obs)
         sd = digamma(self._gamma.T[1] + self._gamma.T[2])
         dgamma1 = digamma(self._gamma.T[1]) - sd
         dgamma2 = np.zeros(self.n_components)
@@ -383,11 +367,9 @@ class DPGMM(GMM):
                 T = np.sum(self._z.T[k])
                 self._a[k] = 2 + T + self.n_features
                 self._B[k] = (T + 1) * np.identity(self.n_features)
-                for i in xrange(self._X.shape[0]):
-                    dif = self._X[i] - self._means[k]
-                    self._B[k] += self._z[i, k] * np.dot(dif.reshape((-1, 1)),
-                                                         dif.reshape((1, -1)))
-                self._B[k] = linalg.pinv(self._B[k])
+                dx = self._X - self._means[k]
+                self._B[k] += np.dot((self._z[:, k] * dx.T), dx)
+                self._B[k] = linalg.inv(self._B[k])
                 self._precs[k] = self._a[k] * self._B[k]
                 self._detB[k] = linalg.det(self._B[k])
                 self._bound_prec[k] = 0.5 * detlog_wishart(self._a[k],
@@ -535,11 +517,11 @@ class DPGMM(GMM):
             corresponds to a single data point.
         n_iter : int, optional
              Maximum number of iterations to perform before convergence.
-       params : string, optional
+        params : string, optional
             Controls which parameters are updated in the training
             process.  Can contain any combination of 'w' for weights,
             'm' for means, and 'c' for covars.  Defaults to 'wmc'.
-       init_params : string, optional
+        init_params : string, optional
             Controls which parameters are updated in the initialization
             process.  Can contain any combination of 'w' for weights,
             'm' for means, and 'c' for covars.  Defaults to 'wmc'.
@@ -548,7 +530,7 @@ class DPGMM(GMM):
 
         ## initialization step
 
-        self._X = np.asanyarray(X)
+        self._X = np.asarray(X)
         if hasattr(self, 'n_features') and self.n_features != self._X.shape[1]:
             raise ValueError('Unexpected number of dimensions, got %s but '
                              'expected %s' % (self._X.shape[1],
@@ -638,7 +620,7 @@ class VBGMM(DPGMM):
     Variational inference for a Gaussian mixture model probability
     distribution. This class allows for easy and efficient inference
     of an approximate posterior distribution over the parameters of a
-    gaussian mixture model with a fixed number of components.
+    Gaussian mixture model with a fixed number of components.
 
     Initialization is with normally-distributed means and identity
     covariance, for proper convergence.
@@ -657,7 +639,7 @@ class VBGMM(DPGMM):
         Real number representing the concentration parameter of
         the dirichlet distribution. Intuitively, the higher the
         value of alpha the more likely the variational mixture of
-        gaussians model will use all components it can. Defaults
+        Gaussians model will use all components it can. Defaults
         to 1.
 
 
@@ -666,49 +648,36 @@ class VBGMM(DPGMM):
     cvtype : string (read-only)
         String describing the type of covariance parameters used by
         the DP-GMM.  Must be one of 'spherical', 'tied', 'diag', 'full'.
+
     n_features : int
         Dimensionality of the Gaussians.
+
     n_components : int (read-only)
         Number of mixture components.
+
     weights : array, shape (`n_components`,)
         Mixing weights for each mixture component.
+
     means : array, shape (`n_components`, `n_features`)
         Mean parameters for each mixture component.
+
     precisions : array
         Precision (inverse covariance) parameters for each mixture
-        component.  The shape depends on `cvtype`:
+        component.  The shape depends on `cvtype`::
             (`n_components`,)                             if 'spherical',
-            (`n_features`, `n_features`)              if 'tied',
+            (`n_features`, `n_features`)                  if 'tied',
             (`n_components`, `n_features`)                if 'diag',
             (`n_components`, `n_features`, `n_features`)  if 'full'
-    converged_ : bool
+
+    `converged_` : bool
         True when convergence was reached in fit(), False
         otherwise.
 
-    Methods
-    -------
-    decode(X)
-        Find most likely mixture components for each point in `X`.
-    eval(X)
-        Compute a lower-bound of the log likelihood of `X` under the model
-        and an approximate posterior distribution over mixture components.
-    fit(X)
-        Estimate the posterior of themodel parameters from `X` using the
-        variational mean-field algorithm.
-    predict(X)
-        Like decode, find most likely mixtures components for each
-        observation in `X`.
-    rvs(n=1)
-        Generate `n` samples from the posterior for the model.
-    score(X)
-        Compute the log likelihood of `X` under the model.
-
-
     See Also
     --------
-    GMM : Finite gaussian mixture model fit with EM
+    GMM : Finite Gaussian mixture model fit with EM
 
-    DPGMM : Ininite gaussian mixture model, using the dirichlet
+    DPGMM : Ininite Gaussian mixture model, using the dirichlet
     process, fit with a variational algorithm
     """
 
@@ -740,6 +709,7 @@ class VBGMM(DPGMM):
         -------
         logprob : array_like, shape (n_samples,)
             Log probabilities of each data point in `obs`
+
         posteriors: array_like, shape (n_samples, n_components)
             Posterior probabilities of each mixture component for each
             observation
@@ -749,7 +719,7 @@ class VBGMM(DPGMM):
             obs = self._X
         else:
             z = np.zeros((obs.shape[0], self.n_components))
-        obs = np.asanyarray(obs)
+        obs = np.asarray(obs)
         p = np.zeros(self.n_components)
         bound = np.zeros(obs.shape[0])
         dg = digamma(self._gamma) - digamma(np.sum(self._gamma))
