@@ -14,7 +14,13 @@ from scipy.spatial.ckdtree import cKDTree
 from .ball_tree import BallTree
 from ..base import BaseEstimator
 from ..metrics import euclidean_distances
-from ..utils import safe_asanyarray, atleast2d_or_csr
+from ..utils import safe_asarray, atleast2d_or_csr
+
+
+def warn_equidistant():
+    msg = ("kneighbors: neighbor k+1 and neighbor k have the same "
+           "distance: results will be dependent on data order.")
+    warnings.warn(msg)
 
 
 def _check_weights(weights):
@@ -64,14 +70,16 @@ class NeighborsBase(BaseEstimator):
     # this can be passed directly to BallTree and cKDTree.  Brute-force will
     # rely on soon-to-be-updated functionality in the pairwise module.
     def _init_params(self, n_neighbors=None, radius=None,
-                     algorithm='auto', leaf_size=30):
+                     algorithm='auto', leaf_size=30,
+                     warn_on_equidistant=True):
         self.n_neighbors = n_neighbors
         self.radius = radius
         self.algorithm = algorithm
         self.leaf_size = leaf_size
+        self.warn_on_equidistant = warn_on_equidistant
 
         if algorithm not in ['auto', 'brute', 'kd_tree', 'ball_tree']:
-            raise ValueError("unrecognized algorithm")
+            raise ValueError("unrecognized algorithm: '%s'" % algorithm)
 
         self._fit_X = None
         self._tree = None
@@ -96,7 +104,7 @@ class NeighborsBase(BaseEstimator):
             self._fit_method = 'kd_tree'
             return self
 
-        X = safe_asanyarray(X)
+        X = safe_asarray(X)
 
         if X.ndim != 2:
             raise ValueError("data type not understood")
@@ -196,15 +204,25 @@ class KNeighborsMixin(object):
         if self._fit_method == 'brute':
             dist = euclidean_distances(X, self._fit_X, squared=True)
             # XXX: should be implemented with a partial sort
-            neigh_ind = dist.argsort(axis=1)[:, :n_neighbors]
+            neigh_ind = dist.argsort(axis=1)
+            if self.warn_on_equidistant and n_neighbors < self._fit_X.shape[0]:
+                ii = np.arange(dist.shape[0])
+                ind_k = neigh_ind[:, n_neighbors - 1]
+                ind_k1 = neigh_ind[:, n_neighbors]
+                if np.any(dist[ii, ind_k] == dist[ii, ind_k1]):
+                    warn_equidistant()
+            neigh_ind = neigh_ind[:, :n_neighbors]
             if return_distance:
                 j = np.arange(neigh_ind.shape[0])[:, None]
                 return np.sqrt(dist[j, neigh_ind]), neigh_ind
             else:
                 return neigh_ind
         elif self._fit_method == 'ball_tree':
-            return self._tree.query(X, n_neighbors,
-                                    return_distance=return_distance)
+            result = self._tree.query(X, n_neighbors,
+                                      return_distance=return_distance)
+            if self.warn_on_equidistant and self._tree.warning_flag:
+                warn_equidistant()
+            return result
         elif self._fit_method == 'kd_tree':
             dist, ind = self._tree.query(X, n_neighbors)
             # kd_tree returns a 1D array for n_neighbors = 1
@@ -259,7 +277,7 @@ class KNeighborsMixin(object):
         --------
         NearestNeighbors.radius_neighbors_graph
         """
-        X = np.asanyarray(X)
+        X = np.asarray(X)
 
         if n_neighbors is None:
             n_neighbors = self.n_neighbors
@@ -448,7 +466,7 @@ class RadiusNeighborsMixin(object):
         --------
         kneighbors_graph
         """
-        X = np.asanyarray(X)
+        X = np.asarray(X)
 
         if radius is None:
             radius = self.radius
@@ -495,7 +513,7 @@ class SupervisedFloatMixin(object):
         y : {array-like, sparse matrix}, shape = [n_samples]
             Target values, array of float values.
         """
-        self._y = np.asanyarray(y)
+        self._y = np.asarray(y)
         return self._fit(X)
 
 
@@ -512,7 +530,7 @@ class SupervisedIntegerMixin(object):
         y : {array-like, sparse matrix}, shape = [n_samples]
             Target values, array of integer values.
         """
-        self._y = np.asanyarray(y)
+        self._y = np.asarray(y)
         return self._fit(X)
 
 
