@@ -1,19 +1,23 @@
-""" Transformers to perform common preprocessing steps.
+"""
+The :mod:`sklearn.preprocessing` module implements various utilities to perform
+common data preprocessing steps (scaling, normalization, etc).
 """
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Mathieu Blondel <mathieu@mblondel.org>
 #          Olivier Grisel <olivier.grisel@ensta.org>
 # License: BSD
-
 import numpy as np
 import scipy.sparse as sp
 
 from ..utils import check_arrays
+from ..utils import warn_if_not_float
 from ..base import BaseEstimator, TransformerMixin
 
-from ._preprocessing import inplace_csr_row_normalize_l1
-from ._preprocessing import inplace_csr_row_normalize_l2
+from ..utils.sparsefuncs import inplace_csr_row_normalize_l1
+from ..utils.sparsefuncs import inplace_csr_row_normalize_l2
+from ..utils.sparsefuncs import inplace_csr_column_scale
+from ..utils.sparsefuncs import mean_variance_axis0
 
 
 def _mean_and_std(X, axis=0, with_mean=True, with_std=True):
@@ -21,7 +25,7 @@ def _mean_and_std(X, axis=0, with_mean=True, with_std=True):
 
     Zero valued std components are reset to 1.0 to avoid NaNs when scaling.
     """
-    X = np.asanyarray(X)
+    X = np.asarray(X)
     Xr = np.rollaxis(X, axis)
 
     if with_mean:
@@ -48,7 +52,7 @@ def scale(X, axis=0, with_mean=True, with_std=True, copy=True):
 
     Parameters
     ----------
-    X : array-like
+    X : array-like or CSR matrix.
         The data to center and scale.
 
     axis : int (0 by default)
@@ -68,6 +72,19 @@ def scale(X, axis=0, with_mean=True, with_std=True, copy=True):
         copy (if the input is already a numpy array or a scipy.sparse
         CSR matrix and if axis is 1).
 
+    Notes
+    -----
+    This implementation will refuse to center scipy.sparse matrices
+    since it would make them non-sparse and would potentially crash the
+    program with memory exhaustion problems.
+
+    Instead the caller is expected to either set explicitly
+    `with_mean=False` (in that case, only variance scaling will be
+    performed on the features of the CSR matrix) or to call `X.toarray()`
+    if he/she expects the materialized dense array to fit in memory.
+
+    To avoid memory copy the caller should pass a CSR matrix.
+
     See also
     --------
     :class:`sklearn.preprocessing.Scaler` to perform centering and
@@ -75,18 +92,34 @@ def scale(X, axis=0, with_mean=True, with_std=True, copy=True):
     :class:`sklearn.pipeline.Pipeline`)
     """
     if sp.issparse(X):
-        raise NotImplementedError(
-            "Scaling is not yet implement for sparse matrices")
-    X = np.asanyarray(X)
-    mean_, std_ = _mean_and_std(
-        X, axis, with_mean=with_mean, with_std=with_std)
-    if copy:
-        X = X.copy()
-    Xr = np.rollaxis(X, axis)
-    if with_mean:
-        Xr -= mean_
-    if with_std:
-        Xr /= std_
+        if with_mean:
+            raise ValueError(
+                "Cannot center sparse matrices: pass `with_mean=False` instead"
+                " See docstring for motivation and alternatives.")
+        if axis != 0:
+            raise ValueError("Can only scale sparse matrix on axis=0, "
+                             " got axis=%d" % axis)
+        warn_if_not_float(X, estimator='The scale function')
+        if not sp.isspmatrix_csr(X):
+            X = X.tocsr()
+            copy = False
+        if copy:
+            X = X.copy()
+        _, var = mean_variance_axis0(X)
+        var[var == 0.0] = 1.0
+        inplace_csr_column_scale(X, 1 / np.sqrt(var))
+    else:
+        X = np.asarray(X)
+        warn_if_not_float(X, estimator='The scale function')
+        mean_, std_ = _mean_and_std(
+            X, axis, with_mean=with_mean, with_std=with_std)
+        if copy:
+            X = X.copy()
+        Xr = np.rollaxis(X, axis)
+        if with_mean:
+            Xr -= mean_
+        if with_std:
+            Xr /= std_
     return X
 
 
@@ -127,10 +160,10 @@ class Scaler(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    mean_ : array of floats with shape [n_features]
+    `mean_` : array of floats with shape [n_features]
         The mean value for each feature in the training set.
 
-    std_ : array of floats with shape [n_features]
+    `std_` : array of floats with shape [n_features]
         The standard deviation for each feature in the training set.
 
     See also
@@ -152,16 +185,34 @@ class Scaler(BaseEstimator, TransformerMixin):
 
         Parameters
         ----------
-        X : array-like with shape [n_samples, n_features]
+        X : array-like or CSR matrix with shape [n_samples, n_features]
             The data used to compute the mean and standard deviation
             used for later scaling along the features axis.
         """
         if sp.issparse(X):
-            raise NotImplementedError(
-                "Scaling is not yet implement for sparse matrices")
-        self.mean_, self.std_ = _mean_and_std(
-            X, axis=0, with_mean=self.with_mean, with_std=self.with_std)
-        return self
+            if self.with_mean:
+                raise ValueError(
+                    "Cannot center sparse matrices: pass `with_mean=False` "
+                    "instead See docstring for motivation and alternatives.")
+            warn_if_not_float(X, estimator=self)
+            copy = self.copy
+            if not sp.isspmatrix_csr(X):
+                X = X.tocsr()
+                copy = False
+            if copy:
+                X = X.copy()
+            self.mean_ = None
+            _, var = mean_variance_axis0(X)
+            self.std_ = np.sqrt(var)
+            self.std_[var == 0.0] = 1.0
+            inplace_csr_column_scale(X, 1 / self.std_)
+            return self
+        else:
+            X = np.asarray(X)
+            warn_if_not_float(X, estimator=self)
+            self.mean_, self.std_ = _mean_and_std(
+                X, axis=0, with_mean=self.with_mean, with_std=self.with_std)
+            return self
 
     def transform(self, X, y=None, copy=None):
         """Perform standardization by centering and scaling
@@ -173,15 +224,26 @@ class Scaler(BaseEstimator, TransformerMixin):
         """
         copy = copy if copy is not None else self.copy
         if sp.issparse(X):
-            raise NotImplementedError(
-                "Scaling is not yet implement for sparse matrices")
-        X = np.asanyarray(X)
-        if copy:
-            X = X.copy()
-        if self.with_mean:
-            X -= self.mean_
-        if self.with_std:
-            X /= self.std_
+            if self.with_mean:
+                raise ValueError(
+                    "Cannot center sparse matrices: pass `with_mean=False` "
+                    "instead See docstring for motivation and alternatives.")
+            warn_if_not_float(X, estimator=self)
+            if not sp.isspmatrix_csr(X):
+                X = X.tocsr()
+                copy = False
+            if copy:
+                X = X.copy()
+            inplace_csr_column_scale(X, 1 / self.std_)
+        else:
+            X = np.asarray(X)
+            warn_if_not_float(X, estimator=self)
+            if copy:
+                X = X.copy()
+            if self.with_mean:
+                X -= self.mean_
+            if self.with_std:
+                X /= self.std_
         return X
 
     def inverse_transform(self, X, copy=None):
@@ -194,15 +256,24 @@ class Scaler(BaseEstimator, TransformerMixin):
         """
         copy = copy if copy is not None else self.copy
         if sp.issparse(X):
-            raise NotImplementedError(
-                "Scaling is not yet implement for sparse matrices")
-        X = np.asanyarray(X)
-        if copy:
-            X = X.copy()
-        if self.with_std:
-            X *= self.std_
-        if self.with_mean:
-            X += self.mean_
+            if self.with_mean:
+                raise ValueError(
+                    "Cannot uncenter sparse matrices: pass `with_mean=False` "
+                    "instead See docstring for motivation and alternatives.")
+            if not sp.isspmatrix_csr(X):
+                X = X.tocsr()
+                copy = False
+            if copy:
+                X = X.copy()
+            inplace_csr_column_scale(X, self.std_)
+        else:
+            X = np.asarray(X)
+            if copy:
+                X = X.copy()
+            if self.with_std:
+                X *= self.std_
+            if self.with_mean:
+                X += self.mean_
         return X
 
 
@@ -246,6 +317,7 @@ def normalize(X, norm='l2', axis=1, copy=True):
         raise ValueError("'%d' is not a supported axis" % axis)
 
     X = check_arrays(X, sparse_format=sparse_format, copy=copy)[0]
+    warn_if_not_float(X, 'The normalize function')
     if axis == 0:
         X = X.T
 
@@ -296,8 +368,8 @@ class Normalizer(BaseEstimator, TransformerMixin):
         copy (if the input is already a numpy array or a scipy.sparse
         CSR matrix).
 
-    Note
-    ----
+    Notes
+    -----
     This estimator is stateless (besides constructor parameters), the
     fit method does nothing but is useful when used in a pipeline.
 
@@ -430,8 +502,14 @@ class Binarizer(BaseEstimator, TransformerMixin):
         return binarize(X, threshold=self.threshold, copy=copy)
 
 
+def _is_label_indicator_matrix(y):
+    return hasattr(y, "shape") and len(y.shape) == 2
+
+
 def _is_multilabel(y):
-    return isinstance(y[0], tuple) or isinstance(y[0], list)
+    return isinstance(y[0], tuple) or \
+           isinstance(y[0], list) or \
+           _is_label_indicator_matrix(y)
 
 
 class LabelBinarizer(BaseEstimator, TransformerMixin):
@@ -454,7 +532,7 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    classes_ : array of shape [n_class]
+    `classes_`: array of shape [n_class]
         Holds the label for each class.
 
     Examples
@@ -476,6 +554,10 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
     array([1, 2, 3])
     """
 
+    def _check_fitted(self):
+        if not hasattr(self, "classes_"):
+            raise ValueError("LabelBinarizer was not fitted yet.")
+
     def fit(self, y):
         """Fit label binarizer
 
@@ -491,8 +573,11 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
         """
         self.multilabel = _is_multilabel(y)
         if self.multilabel:
-            # concatenation of the sub-sequences
-            self.classes_ = np.unique(reduce(lambda a, b: a + b, y))
+            self.indicator_matrix_ = _is_label_indicator_matrix(y)
+            if self.indicator_matrix_:
+                self.classes_ = np.arange(y.shape[1])
+            else:
+                self.classes_ = np.array(sorted(set.union(*map(set, y))))
         else:
             self.classes_ = np.unique(y)
         return self
@@ -513,16 +598,22 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
         -------
         Y : numpy array of shape [n_samples, n_classes]
         """
+        self._check_fitted()
+
+        if self.multilabel or len(self.classes_) > 2:
+            if _is_label_indicator_matrix(y):
+                # nothing to do as y is already a label indicator matrix
+                return y
 
-        if len(self.classes_) == 2:
-            Y = np.zeros((len(y), 1))
-        else:
             Y = np.zeros((len(y), len(self.classes_)))
+        else:
+            Y = np.zeros((len(y), 1))
 
         y_is_multilabel = _is_multilabel(y)
 
         if y_is_multilabel and not self.multilabel:
-            raise ValueError("The object was not fitted with multilabel input!")
+            raise ValueError("The object was not " +
+                    "fitted with multilabel input!")
 
         elif self.multilabel:
             if not _is_multilabel(y):
@@ -560,9 +651,8 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
             Target values.
 
         threshold : float
-            Threshold used to decide whether to assign the positive class or the
-            negative class in the binary case. Use 0.5 when Y contains
-            probabilities.
+            Threshold used in the binary and multi-label cases.
+            Use 0.5 when Y contains probabilities.
 
         Returns
         -------
@@ -570,7 +660,7 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
             Target values. In the multilabel case the nested sequences can
             have variable lengths.
 
-        Note
+        Notes
         -----
         In the case when the binary labels are fractional
         (probabilistic), inverse_transform chooses the class with the
@@ -578,10 +668,18 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
         linear model's decision_function method directly as the input
         of inverse_transform.
         """
+        self._check_fitted()
+
         if self.multilabel:
-            Y = np.array(Y > 0, dtype=int)
-            return [tuple(self.classes_[np.flatnonzero(Y[i])])
-                    for i in range(Y.shape[0])]
+            Y = np.array(Y > threshold, dtype=int)
+            # Return the predictions in the same format as in fit
+            if self.indicator_matrix_:
+                # Label indicator matrix format
+                return Y
+            else:
+                # Lists of tuples format
+                return [tuple(self.classes_[np.flatnonzero(Y[i])])
+                        for i in range(Y.shape[0])]
 
         if len(Y.shape) == 1 or Y.shape[1] == 1:
             y = np.array(Y.ravel() > threshold, dtype=int)
