@@ -1,6 +1,6 @@
 from ..base import ClassifierMixin, RegressorMixin
 from ..linear_model.base import CoefSelectTransformerMixin
-from .base import BaseLibLinear, BaseLibSVM
+from .base import BaseLibLinear, DenseBaseLibSVM
 
 
 class LinearSVC(BaseLibLinear, ClassifierMixin, CoefSelectTransformerMixin):
@@ -22,12 +22,12 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, CoefSelectTransformerMixin):
 
     penalty : string, 'l1' or 'l2' (default='l2')
         Specifies the norm used in the penalization. The 'l2'
-        penalty is the standard used in SVC. The 'l1' leads to coef_
+        penalty is the standard used in SVC. The 'l1' leads to `coef_`
         vectors that are sparse.
 
     dual : bool, (default=True)
         Select the algorithm to either solve the dual or primal
-        optimization problem.
+        optimization problem. Prefer dual=False when n_samples > n_features.
 
     tol: float, optional (default=1e-4)
         Tolerance for stopping criteria
@@ -52,12 +52,20 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, CoefSelectTransformerMixin):
         To lessen the effect of regularization on synthetic feature weight
         (and therefore on the intercept) intercept_scaling has to be increased
 
+    scale_C : bool
+        Scale C with number of samples. It makes the setting of C independent
+        of the number of samples.
+
     Attributes
     ----------
-    `coef_` : array, shape = [n_features] if n_classes == 2 else [n_classes, n_features]
+    `coef_` : array, shape = [n_features] if n_classes == 2 \
+            else [n_classes, n_features]
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `raw_coef_` that \
+        follows the internal memory layout of liblinear.
+
     `intercept_` : array, shape = [1] if n_classes == 2 else [n_classes]
         Constants in decision function.
 
@@ -68,14 +76,14 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, CoefSelectTransformerMixin):
     to have slightly different results for the same input data. If
     that happens, try with a smaller tol parameter.
 
+    **References:**
+    `LIBLINEAR: A Library for Large Linear Classification
+    <http://www.csie.ntu.edu.tw/~cjlin/liblinear/>`__
+
     See also
     --------
     SVC
 
-    References
-    ----------
-    LIBLINEAR -- A Library for Large Linear Classification
-    http://www.csie.ntu.edu.tw/~cjlin/liblinear/
 
     """
 
@@ -83,7 +91,7 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, CoefSelectTransformerMixin):
     pass
 
 
-class SVC(BaseLibSVM, ClassifierMixin):
+class SVC(DenseBaseLibSVM, ClassifierMixin):
     """C-Support Vector Classification.
 
     Parameters
@@ -110,7 +118,7 @@ class SVC(BaseLibSVM, ClassifierMixin):
 
     probability: boolean, optional (default=False)
         Whether to enable probability estimates. This must be enabled prior
-        to calling prob_predict.
+        to calling predict_proba.
 
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
@@ -118,6 +126,13 @@ class SVC(BaseLibSVM, ClassifierMixin):
     tol: float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
+    cache_size: float, optional
+        Specify the size of the kernel cache (in MB)
+
+    scale_C : bool
+        Scale C with number of samples. It makes the setting of C independant
+        of the number of samples.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -136,6 +151,9 @@ class SVC(BaseLibSVM, ClassifierMixin):
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `dual_coef_` and
+        `support_vectors_`
+
     `intercept_` : array, shape = [n_class * (n_class-1) / 2]
         Constants in decision function.
 
@@ -147,8 +165,8 @@ class SVC(BaseLibSVM, ClassifierMixin):
     >>> from sklearn.svm import SVC
     >>> clf = SVC()
     >>> clf.fit(X, y)
-    SVC(C=1.0, coef0=0.0, degree=3, gamma=0.5, kernel='rbf', probability=False,
-      shrinking=True, tol=0.001)
+    SVC(C=1.0, cache_size=200, coef0=0.0, degree=3, gamma=0.5, kernel='rbf',
+      probability=False, scale_C=False, shrinking=True, tol=0.001)
     >>> print clf.predict([[-0.8, -1]])
     [ 1.]
 
@@ -159,13 +177,14 @@ class SVC(BaseLibSVM, ClassifierMixin):
 
     def __init__(self, C=1.0, kernel='rbf', degree=3, gamma=0.0,
                  coef0=0.0, shrinking=True, probability=False,
-                 tol=1e-3):
+                 tol=1e-3, cache_size=200, scale_C=False):
 
-        BaseLibSVM.__init__(self, 'c_svc', kernel, degree, gamma, coef0,
-                         tol, C, 0., 0., shrinking, probability)
+        super(SVC, self).__init__('c_svc', kernel, degree, gamma, coef0, tol,
+                                  C, 0., 0., shrinking, probability,
+                                  cache_size, scale_C)
 
 
-class NuSVC(BaseLibSVM, ClassifierMixin):
+class NuSVC(DenseBaseLibSVM, ClassifierMixin):
     """Nu-Support Vector Classification.
 
     Parameters
@@ -194,7 +213,7 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
 
     probability: boolean, optional (default=False)
         Whether to enable probability estimates. This must be enabled prior
-        to calling prob_predict.
+        to calling predict_proba.
 
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
@@ -202,6 +221,9 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
     tol: float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
+    cache_size: float, optional
+        Specify the size of the kernel cache (in MB)
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -220,26 +242,12 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `dual_coef_` and
+        `support_vectors_`
+
     `intercept_` : array, shape = [n_class * (n_class-1) / 2]
         Constants in decision function.
 
-    Methods
-    -------
-    fit(X, y) : self
-        Fit the model
-
-    predict(X) : array
-        Predict using the model.
-
-    predict_proba(X) : array
-        Return probability estimates.
-
-    predict_log_proba(X) : array
-        Return log-probability estimates.
-
-    decision_function(X) : array
-        Return distance to predicted margin.
-
     Examples
     --------
     >>> import numpy as np
@@ -248,8 +256,8 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
     >>> from sklearn.svm import NuSVC
     >>> clf = NuSVC()
     >>> clf.fit(X, y)
-    NuSVC(coef0=0.0, degree=3, gamma=0.5, kernel='rbf', nu=0.5, probability=False,
-       shrinking=True, tol=0.001)
+    NuSVC(cache_size=200, coef0=0.0, degree=3, gamma=0.5, kernel='rbf', nu=0.5,
+       probability=False, shrinking=True, tol=0.001)
     >>> print clf.predict([[-0.8, -1]])
     [ 1.]
 
@@ -260,14 +268,14 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
 
     def __init__(self, nu=0.5, kernel='rbf', degree=3, gamma=0.0,
                  coef0=0.0, shrinking=True, probability=False,
-                 tol=1e-3):
+                 tol=1e-3, cache_size=200):
 
-        BaseLibSVM.__init__(self, 'nu_svc', kernel, degree, gamma,
-                         coef0, tol, 0., nu, 0.,
-                         shrinking, probability)
+        super(NuSVC, self).__init__('nu_svc', kernel, degree, gamma, coef0,
+                                    tol, 0., nu, 0., shrinking, probability,
+                                    cache_size, scale_C=None)
 
 
-class SVR(BaseLibSVM, RegressorMixin):
+class SVR(DenseBaseLibSVM, RegressorMixin):
     """epsilon-Support Vector Regression.
 
     The free parameters in the model are C and epsilon.
@@ -302,7 +310,7 @@ class SVR(BaseLibSVM, RegressorMixin):
 
     probability: boolean, optional (default=False)
         Whether to enable probability estimates. This must be enabled prior
-        to calling prob_predict.
+        to calling predict_proba.
 
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
@@ -310,6 +318,13 @@ class SVR(BaseLibSVM, RegressorMixin):
     tol: float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
+    cache_size: float, optional
+        Specify the size of the kernel cache (in MB)
+
+    scale_C : bool
+        Scale C with number of samples. It makes the setting of C independant
+        of the number of samples.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -325,6 +340,9 @@ class SVR(BaseLibSVM, RegressorMixin):
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `dual_coef_` and
+        `support_vectors_`
+
     `intercept_` : array, shape = [n_class * (n_class-1) / 2]
         Constants in decision function.
 
@@ -338,8 +356,9 @@ class SVR(BaseLibSVM, RegressorMixin):
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = SVR(C=1.0, epsilon=0.2)
     >>> clf.fit(X, y)
-    SVR(C=1.0, coef0=0.0, degree=3, epsilon=0.2, gamma=0.2, kernel='rbf',
-      probability=False, shrinking=True, tol=0.001)
+    SVR(C=1.0, cache_size=200, coef0=0.0, degree=3, epsilon=0.2, gamma=0.2,
+      kernel='rbf', probability=False, scale_C=False, shrinking=True,
+      tol=0.001)
 
     See also
     --------
@@ -347,11 +366,11 @@ class SVR(BaseLibSVM, RegressorMixin):
     """
     def __init__(self, kernel='rbf', degree=3, gamma=0.0, coef0=0.0,
                  tol=1e-3, C=1.0, epsilon=0.1, shrinking=True,
-                 probability=False):
+                 probability=False, cache_size=200, scale_C=False):
 
-        BaseLibSVM.__init__(self, 'epsilon_svr', kernel, degree,
-                         gamma, coef0, tol, C, 0.0,
-                         epsilon, shrinking, probability)
+        super(SVR, self).__init__('epsilon_svr', kernel, degree, gamma, coef0,
+                                  tol, C, 0., epsilon, shrinking, probability,
+                                  cache_size, scale_C)
 
     def fit(self, X, y, sample_weight=None, **params):
         """
@@ -364,6 +383,9 @@ class SVR(BaseLibSVM, RegressorMixin):
             n_features is the number of features.
         y : array, shape = [n_samples]
             Target values. Array of floating-point numbers.
+        cache_size: float, optional
+            Specify the size of the cache (in MB)
+
 
         Returns
         -------
@@ -371,10 +393,11 @@ class SVR(BaseLibSVM, RegressorMixin):
             Returns self.
         """
         # we copy this method because SVR does not accept class_weight
-        return BaseLibSVM.fit(self, X, y, sample_weight=sample_weight, **params)
+        return super(SVR, self).fit(X, y, sample_weight=sample_weight,
+                                    **params)
 
 
-class NuSVR(BaseLibSVM, RegressorMixin):
+class NuSVR(DenseBaseLibSVM, RegressorMixin):
     """Nu Support Vector Regression.
 
     Similar to NuSVC, for regression, uses a parameter nu to control
@@ -410,7 +433,7 @@ class NuSVR(BaseLibSVM, RegressorMixin):
 
     probability: boolean, optional (default=False)
         Whether to enable probability estimates. This must be enabled prior
-        to calling prob_predict.
+        to calling predict_proba.
 
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
@@ -418,6 +441,13 @@ class NuSVR(BaseLibSVM, RegressorMixin):
     tol: float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
+    cache_size: float, optional
+        Specify the size of the kernel cache (in MB)
+
+    scale_C : bool
+        Scale C with number of samples. It makes the setting of C independant
+        of the number of samples.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -433,6 +463,9 @@ class NuSVR(BaseLibSVM, RegressorMixin):
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `dual_coef_` and
+        `support_vectors_`
+
     `intercept_` : array, shape = [n_class * (n_class-1) / 2]
         Constants in decision function.
 
@@ -446,8 +479,8 @@ class NuSVR(BaseLibSVM, RegressorMixin):
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = NuSVR(C=1.0, nu=0.1)
     >>> clf.fit(X, y)
-    NuSVR(C=1.0, coef0=0.0, degree=3, gamma=0.2, kernel='rbf', nu=0.1,
-       probability=False, shrinking=True, tol=0.001)
+    NuSVR(C=1.0, cache_size=200, coef0=0.0, degree=3, gamma=0.2, kernel='rbf',
+       nu=0.1, probability=False, scale_C=False, shrinking=True, tol=0.001)
 
     See also
     --------
@@ -456,11 +489,12 @@ class NuSVR(BaseLibSVM, RegressorMixin):
 
     def __init__(self, nu=0.5, C=1.0, kernel='rbf', degree=3,
                  gamma=0.0, coef0=0.0, shrinking=True,
-                 probability=False, tol=1e-3):
+                 probability=False, tol=1e-3, cache_size=200,
+                 scale_C=False):
 
-        BaseLibSVM.__init__(self, 'nu_svr', kernel, degree,
-                         gamma, coef0, tol, C, nu,
-                         None, shrinking, probability)
+        super(NuSVR, self).__init__('nu_svr', kernel, degree, gamma, coef0,
+                                    tol, C, nu, None, shrinking, probability,
+                                    cache_size, scale_C=scale_C)
 
     def fit(self, X, y, sample_weight=None, **params):
         """
@@ -480,10 +514,10 @@ class NuSVR(BaseLibSVM, RegressorMixin):
             Returns self.
         """
         # we copy this method because SVR does not accept class_weight
-        return BaseLibSVM.fit(self, X, y, sample_weight=[], **params)
+        return super(NuSVR, self).fit(X, y, sample_weight=[], **params)
 
 
-class OneClassSVM(BaseLibSVM):
+class OneClassSVM(DenseBaseLibSVM):
     """Unsupervised Outliers Detection.
 
     Estimate the support of a high-dimensional distribution.
@@ -518,6 +552,14 @@ class OneClassSVM(BaseLibSVM):
     shrinking: boolean, optional
         Whether to use the shrinking heuristic.
 
+    cache_size: float, optional
+        Specify the size of the kernel cache (in MB)
+
+    scale_C : bool
+        Scale C with number of samples. It makes the setting of C independant
+        of the number of samples.
+
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -533,14 +575,19 @@ class OneClassSVM(BaseLibSVM):
         Weights asigned to the features (coefficients in the primal
         problem). This is only available in the case of linear kernel.
 
+        `coef_` is readonly property derived from `dual_coef_` and
+        `support_vectors_`
+
     `intercept_` : array, shape = [n_classes-1]
         Constants in decision function.
 
     """
     def __init__(self, kernel='rbf', degree=3, gamma=0.0, coef0=0.0,
-                 tol=1e-3, nu=0.5, shrinking=True):
-        BaseLibSVM.__init__(self, 'one_class', kernel, degree, gamma, coef0,
-                             tol, 0.0, nu, 0.0, shrinking, False)
+                 tol=1e-3, nu=0.5, shrinking=True, cache_size=200):
+
+        super(OneClassSVM, self).__init__('one_class', kernel, degree, gamma,
+                                          coef0, tol, 0., nu, 0., shrinking,
+                                          False, cache_size, scale_C=None)
 
     def fit(self, X, class_weight={}, sample_weight=None, **params):
         """
@@ -558,11 +605,11 @@ class OneClassSVM(BaseLibSVM):
             Returns self.
 
         Notes
-        ------
+        -----
         If X is not a C-ordered contiguous array, it is copied.
 
         """
         super(OneClassSVM, self).fit(
             X, [], class_weight=class_weight, sample_weight=sample_weight,
             **params)
-
+        return self
