@@ -5,11 +5,12 @@ from unittest import TestCase
 
 from sklearn.datasets.samples_generator import make_spd_matrix
 from sklearn import hmm
-from sklearn.utils.extmath import logsum
+from sklearn.utils.extmath import logsumexp
 
 
 np.seterr(all='warn')
 
+
 class SeedRandomNumberGeneratorTestCase(TestCase):
     seed = 9
 
@@ -155,7 +156,7 @@ class TestBaseHMM(SeedRandomNumberGeneratorTestCase):
 
         assert_array_almost_equal(hmmposteriors.sum(axis=1), np.ones(nobs))
 
-        norm = logsum(framelogprob, axis=1)[:, np.newaxis]
+        norm = logsumexp(framelogprob, axis=1)[:, np.newaxis]
         gmmposteriors = np.exp(framelogprob - np.tile(norm, (1, n_components)))
         assert_array_almost_equal(hmmposteriors, gmmposteriors)
 
@@ -174,7 +175,7 @@ class TestBaseHMM(SeedRandomNumberGeneratorTestCase):
         # posteriors, not likelihoods).
         viterbi_ll, state_sequence = h.decode([])
 
-        norm = logsum(framelogprob, axis=1)[:, np.newaxis]
+        norm = logsumexp(framelogprob, axis=1)[:, np.newaxis]
         gmmposteriors = np.exp(framelogprob - np.tile(norm, (1, n_components)))
         gmmstate_sequence = gmmposteriors.argmax(axis=1)
         assert_array_equal(state_sequence, gmmstate_sequence)
@@ -184,7 +185,8 @@ class TestBaseHMM(SeedRandomNumberGeneratorTestCase):
         startprob = self.prng.rand(n_components)
         startprob = startprob / startprob.sum()
         transmat = prng.rand(n_components, n_components)
-        transmat /= np.tile(transmat.sum(axis=1)[:, np.newaxis], (1, n_components))
+        transmat /= np.tile(transmat.sum(axis=1)
+                [:, np.newaxis], (1, n_components))
 
         h = self.StubHMM(n_components)
 
@@ -218,6 +220,7 @@ def train_hmm_and_keep_track_of_log_likelihood(hmm, obs, n_iter=1, **kwargs):
 
 prng = np.random.RandomState(10)
 
+
 class GaussianHMMParams(object):
     n_components = 3
     n_features = 3
@@ -227,10 +230,12 @@ class GaussianHMMParams(object):
     transmat /= np.tile(transmat.sum(axis=1)[:, np.newaxis], (1, n_components))
     means = prng.randint(-20, 20, (n_components, n_features))
     covars = {'spherical': (1.0 + 2 * prng.rand(n_components)) ** 2,
-              'tied': (make_spd_matrix(n_features, random_state=0) + np.eye(n_features)),
+              'tied': (make_spd_matrix(n_features, random_state=0)
+                  + np.eye(n_features)),
               'diag': (1.0 + 2 * prng.rand(n_components, n_features)) ** 2,
               'full': np.array(
-                  [make_spd_matrix(n_features, random_state=0) + np.eye(n_features)
+                  [make_spd_matrix(n_features,
+                      random_state=0) + np.eye(n_features)
                    for x in xrange(n_components)])}
     expanded_covars = {'spherical': [np.eye(n_features) * cov
                                      for cov in covars['spherical']],
@@ -317,7 +322,7 @@ class GaussianHMMTester(GaussianHMMParams):
         h = hmm.GaussianHMM(self.n_components, self.cvtype)
         h.startprob = self.startprob
         h.transmat = hmm.normalize(self.transmat
-                                   + np.diag(self.prng.rand(self.n_components)), 1)
+                + np.diag(self.prng.rand(self.n_components)), 1)
         h.means = 20 * self.means
         h.covars = self.covars[self.cvtype]
 
@@ -365,7 +370,7 @@ class GaussianHMMTester(GaussianHMMParams):
         h.startprob = self.startprob
         h.startprob_prior = startprob_prior
         h.transmat = hmm.normalize(self.transmat
-                                   + np.diag(self.prng.rand(self.n_components)), 1)
+                + np.diag(self.prng.rand(self.n_components)), 1)
         h.transmat_prior = transmat_prior
         h.means = 20 * self.means
         h.means_prior = means_prior
