@@ -360,7 +360,7 @@ class _ArpackParams(object):
         num_iter = self.iparam[2]
         try:
             ev, vec = self.extract(True)
-        except ArpackError, err:
+        except ArpackError as err:
             msg = "%s [%s]" % (msg, err)
             ev = np.zeros((0,))
             vec = np.zeros((self.n, 0))
@@ -886,6 +886,7 @@ def _aslinearoperator_with_dtype(m):
         m.dtype = (m * x).dtype
     return m
 
+
 class SpLuInv(LinearOperator):
     """
     SpLuInv:
@@ -906,6 +907,7 @@ class SpLuInv(LinearOperator):
         else:
             return self.M_lu.solve(x)
 
+
 class LuInv(LinearOperator):
     """
     LuInv:
@@ -994,6 +996,7 @@ class IterOpInv(LinearOperator):
                              % (self.ifunc.__name__, info))
         return b
 
+
 def get_inv_matvec(M, symmetric=False, tol=0):
     if isdense(M):
         return LuInv(M).matvec
@@ -1151,12 +1154,6 @@ def eigs(A, k=6, M=None, sigma=None, which='LM', v0=None,
     eigsh : eigenvalues and eigenvectors for symmetric matrix A
     svds : singular value decomposition for a matrix A
 
-    Notes
-    -----
-    This function is a wrapper to the ARPACK [1]_ SNEUPD, DNEUPD, CNEUPD,
-    ZNEUPD, functions which use the Implicitly Restarted Arnoldi Method to
-    find the eigenvalues and eigenvectors [2]_.
-
     Examples
     --------
     Find 6 eigenvectors of the identity matrix:
@@ -1169,8 +1166,14 @@ def eigs(A, k=6, M=None, sigma=None, which='LM', v0=None,
     >>> vecs.shape
     (13, 6)
 
-    References
-    ----------
+    Notes
+    -----
+    This function is a wrapper to the ARPACK [1]_ SNEUPD, DNEUPD, CNEUPD,
+    ZNEUPD, functions which use the Implicitly Restarted Arnoldi Method to
+    find the eigenvalues and eigenvectors [2]_.
+
+    **References**:
+
     .. [1] ARPACK Software, http://www.caam.rice.edu/software/ARPACK/
     .. [2] R. B. Lehoucq, D. C. Sorensen, and C. Yang,  ARPACK USERS GUIDE:
        Solution of Large Scale Eigenvalue Problems by Implicitly Restarted
@@ -1407,8 +1410,9 @@ def eigsh(A, k=6, M=None, sigma=None, which='LM', v0=None,
     >>> vecs.shape
     (13, 6)
 
-    References
-    ----------
+    Notes
+    -----
+    **References**:
     .. [1] ARPACK Software, http://www.caam.rice.edu/software/ARPACK/
     .. [2] R. B. Lehoucq, D. C. Sorensen, and C. Yang,  ARPACK USERS GUIDE:
        Solution of Large Scale Eigenvalue Problems by Implicitly Restarted
@@ -1550,8 +1554,8 @@ def svds(A, k=6, ncv=None, tol=0):
     tol : float, optional
         Tolerance for singular values. Zero (default) means machine precision.
 
-    Note
-    ----
+    Notes
+    -----
     This is a naive implementation using an eigensolver on A.H * A or
     A * A.H, depending on which one is more efficient.
 
