@@ -1,7 +1,12 @@
-"""Compatibility fixes for older version of python, numpy and scipy"""
+"""Compatibility fixes for older version of python, numpy and scipy
+
+If you add content to this file, please give the version of the package
+at which the fixe is no longer needed.
+"""
 # Authors: Emmanuelle Gouillart <emmanuelle.gouillart@normalesup.org>
 #          Gael Varoquaux <gael.varoquaux@normalesup.org>
 #          Fabian Pedregosa <fpedregosa@acm.org>
+#          Lars Buitinck <L.J.Buitinck@uva.nl>
 # License: BSD
 
 import collections
@@ -9,29 +14,27 @@ import numpy as np
 from operator import itemgetter
 
 
-try:
-    from itertools import product
-except ImportError:
-    def product(*args, **kwds):
-        pools = map(tuple, args) * kwds.get('repeat', 1)
-        result = [[]]
-        for pool in pools:
-            result = [x + [y] for x in result for y in pool]
-        for prod in result:
-            yield tuple(prod)
-
-
 try:
     Counter = collections.Counter
 except AttributeError:
-    # Partial replacement for Python 2.7 Counter
+# Partial replacement for Python 2.7 collections.Counter
     class Counter(collections.defaultdict):
-        def __init__(self, **kwargs):
+        def __init__(self, iterable=(), **kwargs):
             super(Counter, self).__init__(int, **kwargs)
+            self.update(iterable)
 
         def most_common(self):
             return sorted(self.iteritems(), key=itemgetter(1), reverse=True)
 
+        def update(self, other):
+            """Adds counts for elements in other"""
+            if isinstance(other, self.__class__):
+                for x, n in other.iteritems():
+                    self[x] += n
+            else:
+                for x in other:
+                    self[x] += 1
+
 
 def _unique(ar, return_index=False, return_inverse=False):
     """A replacement for the np.unique that appeared in numpy 1.4.
@@ -46,7 +49,7 @@ def _unique(ar, return_index=False, return_inverse=False):
             items = sorted(set(ar))
             return np.asarray(items)
         else:
-            ar = np.asanyarray(ar).flatten()
+            ar = np.asarray(ar).flatten()
 
     if ar.size == 0:
         if return_inverse and return_index:
@@ -131,19 +134,6 @@ def qr_economic(A, **kwargs):
         return scipy.linalg.qr(A, econ=True, **kwargs)
 
 
-def arpack_eigsh(A, **kwargs):
-    """Compat function for sparse symmetric eigen vectors decomposition
-
-    Scipy 0.9 renamed eigen_symmetric to eigsh in
-    scipy.sparse.linalg.eigen.arpack
-    """
-    from scipy.sparse.linalg.eigen import arpack
-    if hasattr(arpack, 'eigsh'):
-        return arpack.eigsh(A, **kwargs)
-    else:
-        return arpack.eigen_symmetric(A, **kwargs)
-
-
 def savemat(file_name, mdict, oned_as="column", **kwargs):
     """MATLAB-format output routine that is compatible with SciPy 0.7's.
 
